@@ -1,6 +1,6 @@
 /*******************************************************************************
 *                                                                              *
-*   (C) 1997-2013 by Ernst W. Mayer.                                           *
+*   (C) 1997-2015 by Ernst W. Mayer.                                           *
 *                                                                              *
 *  This program is free software; you can redistribute it and/or modify it     *
 *  under the terms of the GNU General Public License as published by the       *
@@ -33,6 +33,10 @@
 #include "prefetch.h"
 #include "util.h"
 
+#if defined(USE_FGT61) && defined(USE_SSE2)
+	#error USE_FGT61 and USE_SSE2 may not be defined together!
+#endif
+
 #ifdef INCLUDE_PM1
 	#include "gcd_lehmer.h"
 #endif
@@ -54,7 +58,6 @@ uint32	ernstMain
 	int		radix_set,
 	uint32	maxFFT,
 	uint32	iterations,	/* Use to store log2[max factor depth] in TF mode */
-	int		error_checking,
 	uint64	*sh0,	/* Reference/Computed mod-(2^64) residue */
 	uint64	*sh1,	/* Reference/Computed Selfridge/Hurwitz Residue mod 2^35-1 */
 	uint64	*sh2,	/* Reference/Computed Selfridge/Hurwitz Residue mod 2^36-1 */
@@ -62,6 +65,7 @@ uint32	ernstMain
 	double	*runtime
 );
 
+void	print_help(char*option);
 int		cfgNeedsUpdating(char*in_line);
 void	printMlucasErrCode(int retVal);
 uint64	res64	(double a[], int n, const uint64 p, int *nbits, char *hex_res);
@@ -84,9 +88,9 @@ void	bit_reverse_int(int vec[], int n, int nradices, int radix[], int incr, int*
 /* get_fft_radices.c: */
 int		get_fft_radices			(uint32 kblocks, int radix_set, int *nradices, int radix_vec[], int radix_vec_dim);
 void	test_fft_radixtables	(void);
-uint32	get_default_fft_length	(uint32 p);
+uint32	get_default_fft_length	(uint64 p);
 uint32	get_nextlarger_fft_length	(uint32 n);
-uint32	given_N_get_maxP		(uint32 N);
+uint64	given_N_get_maxP		(uint32 N);
 
 /* get_preferred_fft_radix.c: */
 uint32	get_preferred_fft_radix(uint32 kblocks);
@@ -107,7 +111,11 @@ void	radix12_dif_pass1	(double a[], int n);
 void	radix13_dif_pass1	(double a[], int n);
 void	radix14_dif_pass1	(double a[], int n);
 void	radix15_dif_pass1	(double a[], int n);
-void	radix16_dif_pass1	(double a[], int n);
+#ifdef USE_FGT61
+void	radix16_dif_pass1	(double a[], uint64 b[], int n);
+#else
+void	radix16_dif_pass1	(double a[],             int n);
+#endif
 void	radix18_dif_pass1	(double a[], int n);
 void	radix20_dif_pass1	(double a[], int n);
 void	radix22_dif_pass1	(double a[], int n);
@@ -176,7 +184,11 @@ void	radix13_dit_pass1A	(double a[], int n);
 void	radix13_dit_pass1B	(double a[], int n);
 void	radix14_dit_pass1	(double a[], int n);
 void	radix15_dit_pass1	(double a[], int n);
-void	radix16_dit_pass1	(double a[], int n);
+#ifdef USE_FGT61
+void	radix16_dit_pass1	(double a[], uint64 b[], int n);
+#else
+void	radix16_dit_pass1	(double a[],             int n);
+#endif
 void	radix18_dit_pass1	(double a[], int n);
 void	radix20_dit_pass1	(double a[], int n);
 void	radix22_dit_pass1	(double a[], int n);
@@ -232,26 +244,49 @@ void	radix1024_dit_pass1	(double a[], int n);
 void	radix4032_dit_pass1 (double a[], int n);
 void	radix4096_dit_pass1	(double a[], int n);
 
+void	radix4_dif_pass		(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
 void	radix8_dif_pass		(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
-void	radix16_dif_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+#ifdef USE_FGT61
+	void radix16_dif_pass	(double a[], uint64 b[], int n, struct complex rt0[], struct complex rt1[], uint128 mt0[], uint128 mt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+#else
+	void radix16_dif_pass	(double a[],             int n, struct complex rt0[], struct complex rt1[],                               int index[], int nloops, int incr, int init_sse2, int thr_id);
+#endif
 void	radix32_dif_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
-void	radix64_dif_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+//void	radix64_dif_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
 
+void	radix4_dit_pass		(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
 void	radix8_dit_pass		(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
-void	radix16_dit_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+#ifdef USE_FGT61
+	void radix16_dit_pass	(double a[], uint64 b[], int n, struct complex rt0[], struct complex rt1[], uint128 mt0[], uint128 mt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+#else
+	void radix16_dit_pass	(double a[],             int n, struct complex rt0[], struct complex rt1[],                               int index[], int nloops, int incr, int init_sse2, int thr_id);
+#endif
 void	radix32_dit_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
-void	radix64_dit_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
+//void	radix64_dit_pass	(double a[], int n, struct complex rt0[], struct complex rt1[], int index[], int nloops, int incr, int init_sse2, int thr_id);
 
 /* mers_mod_square.c: */
-int		mers_mod_square		(double a[], int arr_scratch[], int n, int ilo, int ihi, uint64 p, uint32 *err_iter, int scrnFlag, double *tdiff);
+#ifdef USE_FGT61
+	int	mers_mod_square		(double a[], uint64 b[], int arr_scratch[], int n, int ilo, int ihi, uint64 p, int scrnFlag, double *tdiff);
+#else
+	int	mers_mod_square		(double a[],             int arr_scratch[], int n, int ilo, int ihi, uint64 p, int scrnFlag, double *tdiff);
+#endif
 
 	/* radix{16|32|64}_wrapper_square.c: */
+#ifdef USE_FGT61
+	void	pair_square  (double *x1, double *y1, double *x2, double *y2, double c, double s,
+						  uint64 *u1, uint64 *v1, uint64 *u2, uint64 *v2, uint64 a, uint64 b);
+#else
 	void	pair_square  (double *x1, double *y1, double *x2, double *y2, double c, double s);
+#endif
 	void	pair_square2A(double *x1, double *y1, double *x2, double *y2, double *x3, double *y3, double *x4, double *y4, double c, double s);
 	void	pair_square2B(double *x1, double *y1, double *x2, double *y2, double *x3, double *y3, double *x4, double *y4, double c, double s);
 
 	void	radix16_wrapper_ini		(int n, int radix0, int iblock, int nradices_prim, int radix_prim[], int i[], int j1[], int j2[], int j2_start[], int k[], int m[], int blocklen[], int blocklen_sum[]);
-	void	radix16_wrapper_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum, int init_sse2, int thr_id);
+#ifdef USE_FGT61
+	void	radix16_wrapper_square	(double a[], uint64 b[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], uint128 mt0[], uint128 mt1[], int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum, int init_sse2, int thr_id);
+#else
+	void	radix16_wrapper_square	(double a[],             int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[],                               int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum, int init_sse2, int thr_id);
+#endif
 
 	void	radix32_wrapper_ini		(int n, int radix0, int iblock, int nradices_prim, int radix_prim[], int i[], int j1[], int j2[], int j2_start[], int k[], int m[], int blocklen[], int blocklen_sum[]);
 	void	radix32_wrapper_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum, int init_sse2, int thr_id);
@@ -259,10 +294,31 @@ int		mers_mod_square		(double a[], int arr_scratch[], int n, int ilo, int ihi, u
 	void	radix64_wrapper_ini		(int n, int radix0, int iblock, int nradices_prim, int radix_prim[], int i[], int j1[], int j2[], int j2_start[], int k[], int m[], int blocklen[], int blocklen_sum[]);
 	void	radix64_wrapper_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum, int init_sse2, int thr_id);
 
+	/* radix{16|32|64}_pairFFT_mul.c: */
+	void	radix16_pairFFT_mul_ini	(int n, int radix0, int iblock, int nradices_prim, int radix_prim[], int i[], int j1[], int j2[], int j2_start[], int k[], int m[], int blocklen[], int blocklen_sum[]);
+	void	pairFFT_mul				(double x[], double y[], double z[], int n, int INIT_ARRAYS, int FORWARD_FFT_ONLY);
+	void	pairFFT_mul_process_chunk(
+				double a[], double ab_mul[], double cd_mul[], int n, struct complex rt0[], struct complex rt1[],
+				int index[], int block_index[], int ii, int nradices_prim, int radix_prim[],
+				int ws_i[], int ws_j1[], int ws_j2[], int ws_j2_start[], int ws_k[], int ws_m[], int ws_blocklen[], int ws_blocklen_sum[],
+				int FORWARD_FFT_ONLY
+			);
+	void	radix16_pairFFT_mul(
+				double uv[], double ab_mul[], double cd_mul[], int n, int radix0, struct complex rt0[], struct complex rt1[],
+				int nradices_prim, int radix_prim[], int ws_i, int ws_j1, int ws_j2, int ws_j2_start, int ws_k, int ws_m, int ws_blocklen, int ws_blocklen_sum,
+				int INIT_ARRAYS, int FORWARD_FFT_ONLY
+			);
+
 /* fermat_mod_square.c: */
-int 	fermat_mod_square	(double a[], int arr_scratch[], int n, int ilo, int ihi, uint64 p, uint32 *err_iter, int scrnFlag, double *tdiff);
+#if 0//def USE_FGT61	**** First get things working for LL case ****
+	int	fermat_mod_square	(double a[], uint64 b[], int arr_scratch[], int n, int ilo, int ihi, uint64 p, int scrnFlag, double *tdiff);
+#else
+	int	fermat_mod_square	(double a[],             int arr_scratch[], int n, int ilo, int ihi, uint64 p, int scrnFlag, double *tdiff);
+#endif
 
 #ifdef MULTITHREAD
+	uint32 parseAffinityTriplet(char*istr);
+	void parseAffinityString(char*istr);
 	/* Multithreaded version must be in form of 1-arg functor */
 	void *mers_process_chunk  (void*targ);
 	void *fermat_process_chunk(void*targ);
@@ -318,14 +374,19 @@ int 	fermat_mod_square	(double a[], int arr_scratch[], int n, int ilo, int ihi,
 	void *cy4032_process_chunk(void*targ);
 	void *cy4096_process_chunk(void*targ);
 #else
-	void mers_process_chunk  (double a[], int arr_scratch[], int n, struct complex rt0[], struct complex rt1[], int index[], int block_index[], int ii, int nradices_prim, int radix_prim[], int ws_i[], int ws_j1[], int ws_j2[], int ws_j2_start[], int ws_k[], int ws_m[], int ws_blocklen[], int ws_blocklen_sum[]);
+  #ifdef USE_FGT61
+	void mers_process_chunk  (double a[], int arr_scratch[], int n, struct complex rt0[], struct complex rt1[], uint128 mt0[], uint128 mt1[], int index[], int block_index[], int ii, int nradices_prim, int radix_prim[], int ws_i[], int ws_j1[], int ws_j2[], int ws_j2_start[], int ws_k[], int ws_m[], int ws_blocklen[], int ws_blocklen_sum[]);
+  #else
+	void mers_process_chunk  (double a[], int arr_scratch[], int n, struct complex rt0[], struct complex rt1[],                               int index[], int block_index[], int ii, int nradices_prim, int radix_prim[], int ws_i[], int ws_j1[], int ws_j2[], int ws_j2_start[], int ws_k[], int ws_m[], int ws_blocklen[], int ws_blocklen_sum[]);
+  #endif
+
 	void fermat_process_chunk(double a[], int arr_scratch[], int n, struct complex rt0[], struct complex rt1[], int index[],                    int ii, int nradices_prim, int radix_prim[]);
 #endif
 
 	/* radix{16|32|64}_dyadic_square.c: */
 	void	radix16_dyadic_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int ii, int nradices_prim, int radix_prim[], int incr, int init_sse2, int thr_id);
 	void	radix32_dyadic_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int ii, int nradices_prim, int radix_prim[], int incr, int init_sse2, int thr_id);
-	void	radix64_dyadic_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int ii, int nradices_prim, int radix_prim[], int incr, int init_sse2, int thr_id);
+//	void	radix64_dyadic_square	(double a[], int arr_scratch[], int n, int radix0, struct complex rt0[], struct complex rt1[], int ii, int nradices_prim, int radix_prim[], int incr, int init_sse2, int thr_id);
 
 /* Each of the final-DIT-pass/propagate-carries/initial-DIF-pass routines comes in both an error-checking and non-error-checking
 version, unless it's one of the later SSE2-only radices, in which at least partial error checking is mandatory:
@@ -341,7 +402,11 @@ version, unless it's one of the later SSE2-only radices, in which at least parti
 	int	radix13_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 	int	radix14_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 	int	radix15_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
-	int radix16_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
+#ifdef USE_FGT61
+	int radix16_ditN_cy_dif1		(double a[], uint64 b[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
+#else
+	int radix16_ditN_cy_dif1		(double a[],             int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
+#endif
 	int	radix18_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 	int	radix20_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 	int	radix22_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter, double *fracmax, uint64 p);
@@ -397,30 +462,4 @@ version, unless it's one of the later SSE2-only radices, in which at least parti
 	int	radix4032_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 	int	radix4096_ditN_cy_dif1		(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter, double *fracmax, uint64 p);
 
-	/* Only define 2nd version of carry routine[s] with ROE checking disabled in non-SSE2 mode, as SSE2 ROE checking is cheap.
-	For large radices (> 36) also define just a single (ROE-enabled) version of the carry routine and use it irrespective of the command-line ROE-check flag value: */
-  #ifndef USE_SSE2
-	int	radix5_ditN_cy_dif1_nochk 	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix6_ditN_cy_dif1_nochk 	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix7_ditN_cy_dif1_nochk 	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int radix8_ditN_cy_dif1_nochk 	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix9_ditN_cy_dif1_nochk 	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix10_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix11_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix12_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix13_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix14_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix15_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int radix16_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix18_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix20_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix22_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix24_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix26_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[],                                             double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix28_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix30_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int radix32_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-	int	radix36_ditN_cy_dif1_nochk	(double a[], int n, int nwt, int nwt_bits, double wt0[], double wt1[], int si[], struct complex rn0[], struct complex rn1[], double base[], double baseinv[], int iter,                  uint64 p);
-  #endif
-
 #endif	/* Mlucas_h_included */
