@@ -1,6 +1,6 @@
 /*******************************************************************************
 *                                                                              *
-*   (C) 1997-2014 by Ernst W. Mayer.                                           *
+*   (C) 1997-2015 by Ernst W. Mayer.                                           *
 *                                                                              *
 *  This program is free software; you can redistribute it and/or modify it     *
 *  under the terms of the GNU General Public License as published by the       *
@@ -33,6 +33,51 @@
 		uint64 q;
 	};
 
+	/*******************************************/
+	/*      Fermat-number test factors:        */
+	/*******************************************/
+
+	// Here interpret the above testFac struct as a minimalist [n,k]-pair format,
+	// where Fn = 2^2^n+1 is the Fermat number and q = k.2^(n+2)+1 the factor:
+	// To check any particular (alleged) factor q of Fn using Pari, use Mod(2,q)^(2^n)+1.
+
+	// Testcases with factors < 2^64 = 18446744073709551616 Must pad each array with a trailing pair of zeros:
+	static const struct testFac ffac64[] =
+	{
+		{ 5,5ull},{ 5,52347ull},
+		{ 6,1071ull},{ 6,262814145745ull},
+		{ 7,116503103764643ull},
+		{ 8,1209889024954ull},
+		{ 9,1184ull},
+		{10,11131ull},{10,1583748ull},
+		{11,39ull},{11,119ull},
+		{12,7ull},{12,1588ull},{12,3892ull},{12,11613415ull},{12,76668221077ull},
+		{13,82731770ull},{13,81294216221684ull},{13,109958186173776ull},
+		{15,9264ull},{15,17753925353ull},
+		{16,3150ull},
+		{17,59251857ull},
+		{18,13ull},	
+		{19,33629ull},{19,308385ull},
+		{21,534689ull},
+		{23,5ull},	
+		{25,193652ull},{25,1522849979ull},{25,16168301139ull}, 
+		{26,286330ull},
+		{27,282030ull},{27,430816215ull},
+		{29,1120049ull},
+		{30,149041ull},{30,255178ull},
+		{32,1479ull},
+		{36,10ull},{36,3759613ull},
+		{38,6ull},{38,2653ull},
+		{39,21ull},	
+		{42,86970ull},
+		{55,29ull},	
+		{0,0ull}
+	};
+
+	/*******************************************/
+	/*      Mersenne-number test factors:      */
+	/*******************************************/
+
 	// Exponent/factor pair arrays for self-tests of bitlengths 63-65. Must pad each array with a trailing pair of zeros.
 	// To check any particular (alleged) factor q of 2^p-1 using Pari, use Mod(2,q)^p-1.
 
@@ -41,8 +86,8 @@
 
 	/* Testcases with factors < 2^63 = 9223372036854775808: */
 	static const struct testFac fac63[] =
-	{
-		{16219289, 7464112771264190017ull},{16230061, 7074289273913679991ull},{16232017, 7411932855077041073ull},{16232537, 6692065114888767967ull},{16233463, 7281938355280163287ull},
+	{	// Start with the 2 smallest factors of MM31:
+		{2147483647,   295257526626031ull},{2147483647, 87054709261955177ull},{16232017, 7411932855077041073ull},{16232537, 6692065114888767967ull},{16233463, 7281938355280163287ull},
 		{16238797, 7160849857727202529ull},{16241383, 8321268976801733153ull},{16247081, 6851345661512506279ull},{16264097, 6651246518968801591ull},{16270871, 8750884836637970639ull},
 		{16273021, 8530948234727688607ull},{16275769, 8656222760818941631ull},{16276951, 8420763231476654791ull},{16287319, 7856663174122645703ull},{16291313, 7319751180156635263ull},
 		{16292609, 9092939873876576857ull},{16304579, 8962807897733335697ull},{16309159, 7820104075051972591ull},{16310741, 7118960435061194119ull},{16315669, 7962842793467390041ull},
@@ -247,7 +292,8 @@
 		{18290411,  684228304469438089ull},{18290659,  563712281110395383ull},{18290927, 3424153274995352719ull},{18291583, 3062407520782516471ull},{18294047, 4139587137772912577ull},
 		{18291773, 2657423207973350447ull},{18292039,  251983731817092089ull},{18293101, 6998952403324571639ull},{18293657,  751539795410988359ull},{18295987, 6428839976520012353ull},
 		{18294259,  602428265212755713ull},{18294721, 7589034719689448759ull},{18295003,  530722364249779991ull},{18295183,  155518178941858441ull},{18296737, 1282191130922981113ull},
-		{18297571, 3288951767406279151ull},{18299003, 4522165993393804423ull},{18299447, 1506874563020357233ull},{0,0ull}
+		{18297571, 3288951767406279151ull},{18299003, 4522165993393804423ull},{18299447, 1506874563020357233ull},{16219289, 7464112771264190017ull},{16230061, 7074289273913679991ull},
+		{0,0ull}
 	};
 
 	/* Testcases with factors < 2^64 = 18446744073709551616: */
