@@ -1,6 +1,6 @@
 /*******************************************************************************
 *                                                                              *
-*   (C) 1997-2012 by Ernst W. Mayer.                                           *
+*   (C) 1997-2017 by Ernst W. Mayer.                                           *
 *                                                                              *
 *  This program is free software; you can redistribute it and/or modify it     *
 *  under the terms of the GNU General Public License as published by the       *
@@ -113,12 +113,12 @@ or the with functions using them (if we declare no _-prepended variables local t
 	MULL32		(x32,y32,lo32)		Lower 32 bits of  64-bit product of x32 and y32 returned in lo32. (Only need in cases where the compiler doesn't properly handle (uint32)(x32 * y32).)
 	MULH32		(x32,y32,hi32)		Upper 32 bits of  64-bit product of x32 and y32 returned in hi32.
 	MUL_LOHI32	(_x,_y,_lo,_hi)		Lower/Upper 32 bits of  64-bit product of _x and _y returned in _lo and _hi, respectively.
+	SQR_LOHI32	(_x,   _lo,_hi)		Lower/Upper 32 bits of  64-bit square  of _x        returned in _lo and _hi, respectively.
 
 	__MULL64	(_x,_y       )	Lower       64 bits of 128-bit product of _x and _y.
 	__MULH64	(_x,_y       )	      Upper 64 bits of 128-bit product of _x and _y.
 	MULL64		(_x,_y,_lo   )	Lower       64 bits of 128-bit product of _x and _y returned in _lo.
 	MULH64		(_x,_y,   _hi)	      Upper 64 bits of 128-bit product of _x and _y returned in          _hi.
-
 	MUL64x32	(_x,_y,_lo,_hi)		Lower 64 and Upper 32 bits of 96-bit product of _x < 2^64 and _y < 2^32.
 	MUL_LOHI64	(_x,_y,_lo,_hi)		Lower/Upper 64 bits of 128-bit product of _x and _y returned in _lo and _hi, respectively.
 	SQR_LOHI64	(_x,   _lo,_hi)		Lower/Upper 64 bits of 128-bit square  of _x        returned in _lo and _hi, respectively.
@@ -128,9 +128,8 @@ or the with functions using them (if we declare no _-prepended variables local t
 #undef MULL32
 #undef MULH32
 #undef MUL_LOHI32
-#undef MUL64x32
-#undef MUL_LOHI64
-#undef SQR_LOHI64
+#undef SQR_LOHI32
+
 #undef __MULL64
 #undef __MULH64
 #undef MULL64
@@ -145,23 +144,31 @@ or the with functions using them (if we declare no _-prepended variables local t
 #endif
 
 #if(defined(CPU_IS_X86)  && (defined(COMPILER_TYPE_GCC) || defined(COMPILER_TYPE_ICC)))
-	#define MUL_LOHI64_SUBROUTINE
-#endif
-
-/* MUL_LOHI64_SUBROUTINE is needed for GCC/PCC build on GNU/Linux  */
-#if defined(COMPILER_TYPE_GCC) && defined(CPU_IS_PPC) && defined(OS_TYPE_LINUX)
-	#define MUL_LOHI64_SUBROUTINE
+//	#define MUL_LOHI64_SUBROUTINE
 #endif
 
 /* Subroutine form, in case compiler has a problem with the macro form:. */
 #ifdef MUL_LOHI64_SUBROUTINE
+  #if defined(VERBOSE_HEADERS) && defined(COMPILER_TYPE_GCC)
+	#warning Defined MUL_LOHI64_SUBROUTINE
+	#warning Using C/32-bit version of MUL_LOHI32 macro
+  #endif
 
-	/* Even though the high output for 64x32-bit is always < 2^32,
-	declare y and hi here as 64-bit ints to allow flexibility for caller:
-	*/
+	#define MUL_LOHI32(_x32,_y32,_lo,_hi)\
+	{\
+		uint64 _x = (uint64)(_x32), _y = (uint64)(_y32);\
+		uint64 _tt = _x * _y;\
+		_lo = (uint32) _tt;\
+		_hi = (uint32)(_tt >> 32);\
+	}
+	#define SQR_LOHI32(_x,   _lo,_hi)	MUL_LOHI32(_x,_x,_lo,_hi)
+	#define __MULL32(x32,y32     )	 ((uint32)(x32)*(uint32)(y32))
+	#define __MULH32(x32,y32     )	(((uint32)(x32)*(uint64)(y32)) >> 32)
+	#define MULL32(  x32,y32,lo32)	lo32 = (uint32)((uint32)(x32)*(uint32)(y32))
+	#define MULH32(  x32,y32,hi32)	hi32 = __MULH32((uint32)(x32),(uint64)(y32))
 	/*
-	uint32	__MULL32	(uint32 x32, uint32 y32);
-	uint32	__MULH32	(uint32 x32, uint32 y32);
+	Even though the high output for 64x32-bit is always < 2^32,
+	declare y and hi here as 64-bit ints to allow flexibility for caller:
 	*/
 	void	MUL64x32(  uint64 x, uint64 y, uint64 *lo, uint64 *hi);
 	void	MUL_LOHI64(uint64 x, uint64 y, uint64 *lo, uint64 *hi);
@@ -298,7 +305,9 @@ or the with functions using them (if we declare no _-prepended variables local t
 
 /* 64-bit x86 (Sun Studio has a bug related to inlines, so we skip it for now) */
 #elif(defined(CPU_IS_X86_64))
-
+  #if defined(VERBOSE_HEADERS) && (defined(COMPILER_TYPE_GCC) || defined(COMPILER_TYPE_NVCC))
+	#warning X86_64-type CPU detected
+  #endif
 	/* Intel C: */
 	#if(defined(COMPILER_TYPE_ICC))
 		#error No 64-bit MUL intrinsics supported for amd64/em64t under Intel C
@@ -357,7 +366,9 @@ or the with functions using them (if we declare no _-prepended variables local t
 
 /* 32-bit X86, Gnu C or MSVC compiler: */
 #elif(defined(CPU_IS_X86))
-
+  #if defined(VERBOSE_HEADERS) && (defined(COMPILER_TYPE_GCC) || defined(COMPILER_TYPE_NVCC))
+	#warning X86_32-type CPU detected
+  #endif
 	#if(defined(COMPILER_TYPE_GCC) || defined(COMPILER_TYPE_SUNC))
 
 		/* The 32x32->64 MUL and 32-bit pairwise ADD macros are adapted from GMP longlong.h: */
@@ -384,12 +395,223 @@ or the with functions using them (if we declare no _-prepended variables local t
 		#define MUL_LOHI32(_x,_y,_lo,_hi)\
 		  __asm__ ("mull %3"							\
 			   : "=a" (_lo), "=d" (_hi)					\
-			   : "%0" ((uint32)(_x)), "rm" ((uint32)(_y)))
+			   : "%0" (_x), "rm" (_y))
 
 		#define MULH32(_x,_y,_hi)\
+		  ({ uint32 _lo;	\
 		  __asm__ ("mull %3"							\
-			   : "=d" (_hi)					\
-			   : "%0" ((uint32)(_x)), "rm" ((uint32)(_y)))
+			   : "=a" (_lo), "=d" (_hi)					\
+			   : "%0" (_x), "rm" (_y)); })
+
+		#define __MULH32(_x, _y)	\
+		  ({ uint32 _lo, _hi;		\
+		  __asm__("mull %3" : "=a" (_lo), "=d" (_hi) : "%0" (_x), "rm" (_y)); _hi; })
+
+		#define __MULL32(x32,y32     )	                ((uint32)(x32)*(uint32)(y32))
+		#define MULL32(  x32,y32,lo32)	lo32 = (uint32) ((uint32)(x32)*(uint32)(y32))
+		#define SQR_LOHI32(_x,   _lo,_hi)	MUL_LOHI32(_x,_x,_lo,_hi)
+
+		/* Low 64 bits of product of uint64 inputs _x and _y returned in uint64 _lo */
+		#define	MULL64(_x,_y,_lo)\
+		{\
+			union {\
+				uint32 u32[2];	/* IA32 is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _a,_b,_c,_d;\
+			uint32 _lo32,_hi32;\
+		\
+			_ux.u64 = _x;\
+			_a = _ux.u32[0];	/* x_lo */\
+			_b = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			_c = _uy.u32[0];	/* y_lo */\
+			_d = _uy.u32[1];	/* y_hi */\
+			MUL_LOHI32(_a,_c,_lo32,_hi32);	\
+			_hi32 += _a*_d + _b*_c;	\
+		\
+			_ux.u32[0] = _lo32;\
+			_ux.u32[1] = _hi32;\
+			_lo= _ux.u64;\
+		}
+
+		/* 64x32=>96-bit product algorithm:
+		represent the inputs as x = a + b*2^32, y = c ( < 2^32), then do 4
+		32-bit MULs and a bunch of add-with-carries to get x*y = b*c*2^32 + a*c .
+
+		On the IA32, we do 32x64-bit via 2 32x32-bit MULs (2 mull).
+
+		Even though the high output for 64x32-bit is always < 2^32,
+		assume _y and _hi here are 64-bit ints to allow flexibility for caller.
+		*/
+		#define MUL64x32(_x, _y,_lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* IA32 is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _a,_b,_c;\
+			uint32 _lo32,_md32,_hi32;\
+			uint32 _bclo;\
+		\
+			_ux.u64 = _x;\
+			_a = _ux.u32[0];	/* x_lo */\
+			_b = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			_c = _uy.u32[0];	/* y_lo */\
+		\
+			MUL_LOHI32(_a,_c,_lo32,_md32);	\
+			MUL_LOHI32(_b,_c,_bclo,_hi32);	\
+		\
+		{\
+		  __asm__ volatile (\
+			"movl	%[_bclo], %%eax		\n\t"\
+			"addl	%%eax	,%[_md32]	\n\t"/* (0) md32 + (b*c)_lo                     , result (middle 32 bits) in md32, carryout in CF bit. */\
+			"movl	%[_hi32], %%eax		\n\t"\
+			"adcl	$0	 	, %%eax		\n\t"/* (1)    0 + (b*c)_hi + (carryin from (0)), result ( upper 32 bits) in hi32, carryout in CF bit - should be zero! */\
+			"movl	%%eax	,%[_hi32]	\n\t"/* Move high result to hi32 */	\
+			: /* outputs: none */\
+			: [_bclo] "m" (_bclo)	/* All inputs from memory/register here */\
+			 ,[_md32] "m" (_md32)	\
+			 ,[_hi32] "m" (_hi32)	\
+			: "cc","memory","eax"	/* Clobbered registers */\
+			);\
+		}\
+		\
+			_ux.u32[0] = _lo32;\
+			_ux.u32[1] = _md32;\
+			_lo= _ux.u64;\
+			_uy.u32[0] = _hi32;\
+			_uy.u32[1] = 0;\
+			_hi= _uy.u64;\
+		}
+
+		#define MUL_LOHI64(_x,_y,_lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* IA32 is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 ah_,al_,bh_,bl_;\
+			uint32 hahbh_,hahbl_,halbh_,halbl_,lahbh_,lahbl_,lalbh_,lalbl_;\
+			uint32 sumhh_,sumhl_,sumlh_;\
+		\
+			_ux.u64 = _x;\
+			al_ = _ux.u32[0];	/* x_lo */\
+			ah_ = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			bl_ = _uy.u32[0];	/* y_lo */\
+			bh_ = _uy.u32[1];	/* y_hi */\
+		\
+		/* EWM: for 32-bit unsigned inputs these have a nominal latency of ?? cycles. */\
+		/* For each MUL output we include the order in which it is used in the add/carry */\
+		/* code below (6 steps, labeled (0)-(5)), in order to properly schedule the MUL: */\
+			MUL_LOHI32(ah_,bh_,lahbh_,hahbh_);	/* (x_hi*y_hi)l,h 4,2 */\
+			MUL_LOHI32(al_,bh_,lalbh_,halbh_);	/* (x_lo*y_hi)l,h 0,1 */\
+			MUL_LOHI32(ah_,bl_,lahbl_,hahbl_);	/* (x_hi*y_lo)l,h 3,1 */\
+			MUL_LOHI32(al_,bl_,lalbl_,halbl_);	/* (x_lo*y_lo)l,h -,0 */\
+		/* Bits   0- 31 : lalbl                           */\
+		/* Bits  32- 63 :*halbl + lahbl +*lalbh   (sumlh) */\
+		/* Bits  64- 95 :*hahbl +*halbh + lahbh   (sumhl) */\
+		/* Bits  96-127 : hahbh                   (sumhh) */\
+		{\
+		  __asm__ volatile (\
+			"movl	%[halbl_], %%eax	\n\t"\
+			"addl	%[lalbh_], %%eax	\n\t"\
+			"movl	%%eax	,%[sumlh_]	\n\t"/* Move high result to hi32 */	\
+			"movl	%[hahbl_], %%eax	\n\t"\
+			"adcl	%[halbh_], %%eax	\n\t"\
+			"movl	%%eax	,%[sumhl_]	\n\t"/* Move high result to hi32 */	\
+			"movl	%[hahbh_], %%eax	\n\t"\
+			"adcl	$0	 	, %%eax		\n\t"/* (1)    0 + (b*c)_hi + (carryin from (0)), result ( upper 32 bits) in hi32, carryout in CF bit - should be zero! */\
+			"movl	%%eax	,%[sumhh_]	\n\t"/* Move high result to hi32 */	\
+			"movl	%[lahbl_], %%eax	\n\t"\
+			"addl	%%eax	,%[sumlh_]	\n\t"\
+		/*	"addl	%[sumlh_], %%eax	\n\t"*/\
+		/*	"movl	%%eax	,%[sumlh_]	\n\t"*/\
+			"movl	%[lahbh_], %%eax	\n\t"\
+			"adcl	%%eax	,%[sumhl_]	\n\t"\
+		/*	"adcl	%[sumhl_], %%eax	\n\t"*/\
+		/*	"movl	%%eax	,%[sumhl_]	\n\t"*/\
+			"adcl	$0	,%[sumhh_]	\n\t"\
+		/*	"movl	$0, %%eax	\n\t"*/\
+		/*	"adcl	%[sumhh_], %%eax	\n\t"*/\
+		/*	"movl	%%eax	,%[sumhh_]	\n\t"*/\
+			: /* outputs: none */\
+			: [hahbh_] "m" (hahbh_)	/* All inputs from memory/register here */\
+			 ,[hahbl_] "m" (hahbl_)	\
+			 ,[halbh_] "m" (halbh_)	\
+			 ,[halbl_] "m" (halbl_)	\
+			 ,[lahbh_] "m" (lahbh_)	\
+			 ,[lahbl_] "m" (lahbl_)	\
+			 ,[lalbh_] "m" (lalbh_)	\
+			 ,[sumhh_] "m" (sumhh_)	\
+			 ,[sumhl_] "m" (sumhl_)	\
+			 ,[sumlh_] "m" (sumlh_)	\
+			: "cc","memory","eax"	/* Clobbered registers */\
+			);\
+		}\
+			_ux.u32[0] = sumhl_;\
+			_ux.u32[1] = sumhh_;\
+			_hi= _ux.u64;\
+			_uy.u32[0] = lalbl_;\
+			_uy.u32[1] = sumlh_;\
+			_lo= _uy.u64;\
+		}
+
+		#define SQR_LOHI64(_x,_lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 ah_,al_;\
+			uint32 hahah_,lahah_,hahal_,lahal_,halal_,lalal_;\
+			uint32 sumhh_,sumhl_,sumlh_;\
+		\
+			_ux.u64 = _x;\
+			al_ = _ux.u32[0];	/* x_lo */\
+			ah_ = _ux.u32[1];	/* x_hi */\
+		\
+			MUL_LOHI32(ah_,ah_,lahah_,hahah_);	/* <64:127> */\
+			MUL_LOHI32(ah_,al_,lahal_,hahal_);	/* <32: 95> */\
+			MUL_LOHI32(al_,al_,lalal_,halal_);	/* < 0: 63> */\
+			/* 2*<lahal,hahal>: */\
+			sumlh_ = lahal_ << 1;\
+			sumhl_ =(hahal_ << 1) | (lahal_>> 31);\
+			sumhh_ = hahah_ + (hahal_>> 31);\
+		{\
+		  __asm__ volatile (\
+			"movl	%[halal_], %%eax	\n\t"\
+			"addl	%[sumlh_], %%eax	\n\t"\
+			"movl	%%eax	,%[sumlh_]	\n\t"\
+			"movl	%[lahah_], %%eax	\n\t"\
+			"adcl	%[sumhl_], %%eax	\n\t"\
+			"movl	%%eax	,%[sumhl_]	\n\t"\
+			"adcl	$0	,%[sumhh_]	\n\t"\
+		/*	"movl	$0, %%eax	\n\t"*/\
+		/*	"adcl	%[sumhh_], %%eax	\n\t"*/\
+		/*	"movl	%%eax	,%[sumhh_]	\n\t"*/\
+			: /* outputs: none */\
+			: [halal_] "m" (halal_)	/* All inputs from memory/register here */\
+			 ,[lahah_] "m" (lahah_)	\
+			 ,[sumhh_] "m" (sumhh_)	\
+			 ,[sumhl_] "m" (sumhl_)	\
+			 ,[sumlh_] "m" (sumlh_)	\
+			: "cc","memory","eax"	/* Clobbered registers */\
+			);\
+		}\
+			_ux.u32[0] = sumhl_;\
+			_ux.u32[1] = sumhh_;\
+			_hi = _ux.u64;\
+			_uy.u32[0] = lalal_;\
+			_uy.u32[1] = sumlh_;\
+			_lo = _uy.u64;\
+		}
 
 	#elif(defined(COMPILER_TYPE_MSVC))
 
@@ -420,6 +642,7 @@ or the with functions using them (if we declare no _-prepended variables local t
 			__asm	mov	_lo, eax		\
 			__asm	mov	_hi, edx		\
 		}
+		#define SQR_LOHI32(_x,   _lo,_hi)	MUL_LOHI32(_x,_x,_lo,_hi)
 
 		#define MULH32(_x,_y,_hi)\
 		{\
@@ -539,7 +762,6 @@ or the with functions using them (if we declare no _-prepended variables local t
 			__asm	mov	eax, hahbh_	\
 			__asm	adc	eax, 0		/* (2)     0 + hahbh + (carryin from sumhl), result (partial sum) in sumhh,carryout in CF) bit - should be zero! */\
 			__asm	mov	sumhh_, eax	\
-		\
 			__asm	mov	eax, lahbl_	\
 			__asm	add	sumlh_, eax	/* (3) sumlh + lahbl                       , result (sumlh), carryout in CF bit. */\
 			__asm	mov	eax, lahbh_	\
@@ -609,6 +831,8 @@ or the with functions using them (if we declare no _-prepended variables local t
 			uint64 _tt;					\
 			MUL_LOHI64((_x), (_y), _tt, _hi);	\
 		}
+		#define	__MULL64(_x, _y)	({ uint64 _lo; MULL64((_x), (_y), _lo); _lo; })
+		#define	__MULH64(_x, _y)	({ uint64 _lo; MULH64((_x), (_y), _lo); _lo; })
 
 /* Itanium: */
 #elif(defined( CPU_IS_IA64 ))
@@ -1084,6 +1308,10 @@ or the with functions using them (if we declare no _-prepended variables local t
 
 #elif !defined(MUL_LOHI64_SUBROUTINE)	/* User compile-time override flag to force subroutine form rather than macro: */
 
+  #ifdef COMPILER_TYPE_GCC
+   #ifdef VERBOSE_HEADERS
+	#warning Using generic C/32-bit version of MUL64 macros
+   #endif
 	#define __MULL32(x32,y32     )                   ((uint32)(x32)*(uint32)(y32))
 	#define __MULH32(x32,y32     )                  (((uint32)(x32)*(uint64)(y32)) >> 32)
 	#define MULL32(  x32,y32,lo32)	lo32 = (uint32) ((uint32)(x32)*(uint32)(y32))
@@ -1108,7 +1336,7 @@ or the with functions using them (if we declare no _-prepended variables local t
 	Even though the high output for 64x32-bit is always < 2^32,
 	assume _y and _hi here are 64-bit ints to allow flexibility for caller.
 	*/
-  #if EWM_DEBUG
+   #if EWM_DEBUG
 	#define  MUL64x32(_x, _y,_lo,_hi)\
 	{\
 		char s0[21],s1[21];\
@@ -1132,7 +1360,7 @@ or the with functions using them (if we declare no _-prepended variables local t
 			ASSERT(HERE, 0,"0");\
 		}\
 	}
-  #else
+   #else
 	#define  MUL64x32(_x, _y,_lo,_hi)\
 	{\
 		uint64 _t;\
@@ -1144,7 +1372,7 @@ or the with functions using them (if we declare no _-prepended variables local t
 		_lo +=  _t;\
 		_hi += (_lo < _t);\
 	}
-  #endif
+   #endif
 
 	/* Generic 128-bit product macros: represent the inputs as
 	x = a + b*2^32, y = c + d*2^32, and then do 4 MULs and a bunch of
@@ -1201,6 +1429,520 @@ or the with functions using them (if we declare no _-prepended variables local t
 		MUL_LOHI64((_x), (_y), _tt, _hi);	\
 	}
 
+  #elif defined(COMPILER_TYPE_NVCC)
+
+	#define MULL32(	 _x,_y, _lo     )	_lo =          (_x)*(_y)
+	#define MULH32(  _x,_y,      _hi)	_hi = __umulhi((_x),(_y))
+	#define __MULL32(_x, _y)  ({ uint32 _lo =          (_x)*(_y) ; _lo; })
+	#define __MULH32(_x, _y)  ({ uint32 _hi = __umulhi((_x),(_y)); _hi; })
+	#define MUL_LOHI32(_x,_y,_lo,_hi)\
+	{\
+		_lo =          (_x)*(_y) ;\
+		_hi = __umulhi((_x),(_y));\
+	}
+
+   #if 0	// My hand-rolled 64-bit-int emulations below are faster than NVCC's emulations-via-intrinsics:
+
+	#warning Using CUDA intrinsic MUL64 macros
+	#define MUL_LOHI64(_x,_y,_lo,_hi){uint64 _t = (_x)*(_y); _hi = __umul64hi((_x), (_y));	_lo = _t;}
+	#define MUL64x32(  _x,_y,_lo,_hi)	MUL_LOHI64(_x,(uint64)_y,_lo,_hi)
+	#define SQR_LOHI64(_x,   _lo,_hi)	MUL_LOHI64(_x,_x,_lo,_hi)
+	#define	__MULL64(	 _x,_y      )	           (_x)*(_y)
+	#define	__MULH64(	 _x,_y      )	__umul64hi((_x),(_y))
+	#define MULL64(	 _x,_y, _lo     )	_lo =            (_x)*(_y)
+	#define MULH64(  _x,_y,      _hi)	_hi = __umul64hi((_x),(_y))
+
+   #else
+
+	// These single-PTX-instructions inline-ASMs are taken staright from Oliver Weihe's MFAKTC my_intrinsics.h file:
+	// R = A + B: carryin =  no; carryout = yes:
+	__device__ static unsigned int __add_cc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("add.cc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+	// R = A + B: carryin = yes; carryout = yes:
+	__device__ static unsigned int __addc_cc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("addc.cc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+	// R = A + B: carryin = yes; carryout =  no:
+	__device__ static unsigned int __addc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("addc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+
+	// R = A - B: carryin =  no; carryout = yes:
+	__device__ static unsigned int __sub_cc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("sub.cc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+	// R = A - B: carryin = yes; carryout = yes:
+	__device__ static unsigned int __subc_cc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("subc.cc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+	// R = A - B: carryin = yes; carryout =  no:
+	__device__ static unsigned int __subc(unsigned int a, unsigned int b)
+	{
+	  unsigned int r;
+	  asm("subc.u32 %0, %1, %2;" : "=r" (r) : "r" (a) , "r" (b));
+	  return r;
+	}
+
+	// R = (A * B).lo + C: carryin =  no; carryout =  no:
+	__device__ static unsigned int __umad32(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("mad.lo.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).lo + C: carryin =  no; carryout = yes:
+	__device__ static unsigned int __umad32_cc(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("mad.lo.cc.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).lo + C: carryin = yes; carryout =  no:
+	__device__ static unsigned int __umad32c(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("madc.lo.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).lo + C: carryin = yes; carryout = yes:
+	__device__ static unsigned int __umad32c_cc(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("madc.lo.cc.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+
+	// R = (A * B).hi + C: carryin =  no; carryout =  no:
+	__device__ static unsigned int __umad32hi(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("mad.hi.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).hi + C: carryin =  no; carryout = yes:
+	__device__ static unsigned int __umad32hi_cc(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("mad.hi.cc.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).hi + C: carryin = yes; carryout =  no:
+	__device__ static unsigned int __umad32hic(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("madc.hi.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+	// R = (A * B).hi + C: carryin = yes; carryout = yes:
+	__device__ static unsigned int __umad32hic_cc(unsigned int a, unsigned int b, unsigned int c)
+	{
+	  unsigned int r;
+	  asm("madc.hi.cc.u32 %0, %1, %2, %3;" : "=r" (r) : "r" (a) , "r" (b), "r" (c));
+	  return r;
+	}
+
+	/* Low 64 bits of product of uint64 inputs _x and _y returned in uint64 _lo */
+	#define	MULL64(_x,_y,_lo)\
+	{\
+		union {\
+			uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+			uint64 u64;\
+		} _ux,_uy;\
+	\
+		uint32 _a,_b,_c,_d;\
+		uint32 _lo32,_hi32;\
+	\
+		_ux.u64 = _x;\
+		_a = _ux.u32[0];	/* x_lo */\
+		_b = _ux.u32[1];	/* x_hi */\
+		_uy.u64 = _y;\
+		_c = _uy.u32[0];	/* y_lo */\
+		_d = _uy.u32[1];	/* y_hi */\
+		MUL_LOHI32(_a,_c,_lo32,_hi32);\
+		_hi32 += _a*_d + _b*_c;	\
+	\
+		_ux.u32[0] = _lo32;\
+		_ux.u32[1] = _hi32;\
+		_lo= _ux.u64;\
+	}
+
+	/* 64x32=>96-bit product algorithm:
+	represent the inputs as x = a + b*2^32, y = c ( < 2^32), then do 4
+	32-bit MULs and a bunch of add-with-carries to get x*y = b*c*2^32 + a*c .
+
+	On the IA32, we do 32x64-bit via 2 32x32-bit MULs (2 mull).
+
+	Even though the high output for 64x32-bit is always < 2^32,
+	assume _y and _hi here are 64-bit ints to allow flexibility for caller.
+	*/
+	#define MUL64x32(_x, _y,_lo,_hi)\
+	{\
+		union {\
+			uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+			uint64 u64;\
+		} _ux,_uy;\
+	\
+		uint32 _a,_b,_c;\
+		uint32 _lo32,_md32,_hi32;\
+		uint32 _bclo,cy;\
+	\
+		_ux.u64 = _x;\
+		_a = _ux.u32[0];	/* x_lo */\
+		_b = _ux.u32[1];	/* x_hi */\
+		_uy.u64 = _y;\
+		_c = _uy.u32[0];	/* y_lo */\
+	\
+		MUL_LOHI32(_a,_c,_lo32,_md32);	\
+		MUL_LOHI32(_b,_c,_bclo,_hi32);	\
+	\
+		_md32 += _bclo;	\
+		cy = _md32 < _bclo;	/* (0) md32 + (b*c)_lo                     , result (middle 32 bits) in md32, carryout in CY. */\
+		_hi32 += cy;		/* (1)    0 + (b*c)_hi + (carryin from (0)), result ( upper 32 bits) in hi32, carryout in CY - should be zero! [In fact assume so.] */\
+	\
+		_ux.u32[0] = _lo32;\
+		_ux.u32[1] = _md32;\
+		_lo= _ux.u64;\
+		_uy.u32[0] = _hi32;\
+		_uy.u32[1] = 0;\
+		_hi= _uy.u64;\
+	}
+
+	#if 1	// PTX-code version of MUL_LOHI64:
+
+		#define MUL_LOHI64(_x,_y,_lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _al,_ah,_bl,_bh;\
+			uint32 _r0,_r1,_r2,_r3;\
+		\
+			_ux.u64 = _x;\
+			_al = _ux.u32[0];	/* x_lo */\
+			_ah = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			_bl = _uy.u32[0];	/* y_lo */\
+			_bh = _uy.u32[1];	/* y_hi */\
+		/* Old 1-line-at-a-time inline-ASM version: */\
+		/*	MULL32(_al,_bl,_r0);				// r0 =(_al*_bl).lo   , no carry-out */\
+		/*	MULH32(_al,_bl,_r1);				// r1 =(_al*_bl).hi   , no carry-out */\
+		/*	_r1 = __umad32_cc(_ah,_bl,_r1);		// r1+=(_ah*_bl).lo   , may carry-out */\
+		/*	_r2 = __umad32hic(_ah,_bl,  0);		// r2 =(_ah*_bl).hi+cy, no carry-out */\
+		/*	_r1 = __umad32_cc(_al,_bh,_r1);		// r1+=(_al*_bh).lo   , may carry-out */\
+		/*	_r2 = __umad32hic_cc(_al,_bh,_r2);	// r2+=(_al*_bh).hi+cy, may carry-out */\
+		/*	_r3 = __addc(  0,  0);				// r3 = cy, no carry-out */\
+		/*	_r2 = __umad32_cc(_ah,_bh,_r2);		// r2+=(_ah*_bh).lo   , may carry-out */\
+		/*	_r3 = __umad32hic(_ah,_bh,_r3);		// r3+=(_ah*_bh).hi+cy */\
+		/* Using straight PTX inline-ASM: */\
+		asm volatile (\
+			"mul.lo.u32		%0,%4,%6;		\n\t"/* r0 =(r4*r6).lo   , no carry-out */\
+			"mul.hi.u32		%1,%4,%6;		\n\t"/* r1 =(r4*r6).hi   , no carry-out */\
+			"mad.lo.cc.u32	%1,%5,%6,%1;	\n\t"/* r1+=(r5*r6).lo   , may carry-out */\
+			"madc.hi.u32	%2,%5,%6, 0;	\n\t"/* r2 =(r5*r6).hi+cy, no carry-out */\
+			"mad.lo.cc.u32	%1,%4,%7,%1;	\n\t"/* r1+=(r4*r7).lo   , may carry-out */\
+			"madc.hi.cc.u32	%2,%4,%7,%2;	\n\t"/* r2+=(r4*r7).hi+cy, may carry-out */\
+			"addc.u32		%3, 0, 0;		\n\t"/* r3 = cy, no carry-out */\
+			"mad.lo.cc.u32	%2,%5,%7,%2;	\n\t"/* r2+=(r5*r7).lo   , may carry-out */\
+			"madc.hi.u32	%3,%5,%7,%3;	\n\t"/* r3+=(r5*r7).hi+cy */\
+		  : "=r" (_r0) , "=r" (_r1) "=r" (_r2) , "=r" (_r3) : "r" (_al), "r" (_ah) , "r" (_bl), "r" (_bh));\
+		\
+			_ux.u32[0] = _r0;\
+			_ux.u32[1] = _r1;\
+			_lo= _ux.u64;\
+			_uy.u32[0] = _r2;\
+			_uy.u32[1] = _r3;\
+			_hi= _uy.u64;\
+		}
+
+		#define SQR_LOHI64(_x,   _lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _al,_ah,_bl,_bh;\
+			uint32 _r0,_r1,_r2,_r3;\
+		\
+			_ux.u64 = _x;\
+			_al = _ux.u32[0];	/* x_lo */\
+			_ah = _ux.u32[1];	/* x_hi */\
+		\
+		asm volatile (\
+			"mul.lo.u32		%0,%4,%4;		\n\t"/* r0 =(r4*r4).lo   ,  no carry-out */\
+			"mul.hi.u32		%1,%4,%4;		\n\t"/* r1 =(r4*r4).hi   ,  no carry-out */\
+			"mad.lo.cc.u32	%1,%4,%5,%1;	\n\t"/* r1+=(r4*r5).lo   , may carry-out */\
+			"madc.hi.u32	%2,%4,%5, 0;	\n\t"/* r2 =(r4*r5).hi+cy,  no carry-out */\
+			"mad.lo.cc.u32	%1,%4,%5,%1;	\n\t"/* r1+=(r4*r5).lo   , may carry-out */\
+			"madc.hi.cc.u32	%2,%4,%5,%2;	\n\t"/* r2+=(r4*r5).hi+cy, may carry-out */\
+			"addc.u32		%3, 0, 0;		\n\t"/* r3 = cy, no carry-out */\
+			"mad.lo.cc.u32	%2,%5,%5,%2;	\n\t"/* r2+=(r5*r5).lo   , may carry-out */\
+			"madc.hi.u32	%3,%5,%5,%3;	\n\t"/* r3+=(r5*r5).hi+cy */\
+		  : "=r" (_r0) , "=r" (_r1) "=r" (_r2) , "=r" (_r3) : "r" (_al), "r" (_ah));\
+		\
+			_ux.u32[0] = _r0;\
+			_ux.u32[1] = _r1;\
+			_lo= _ux.u64;\
+			_uy.u32[0] = _r2;\
+			_uy.u32[1] = _r3;\
+			_hi= _uy.u64;\
+		}
+
+		#define MULH64_TEST(_x, _y,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _al,_ah,_bl,_bh;\
+			uint32 _r0,_r1,_r2,_r3;\
+		\
+			_ux.u64 = _x;\
+			_al = _ux.u32[0];	/* x_lo */\
+			_ah = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			_bl = _uy.u32[0];	/* y_lo */\
+			_bh = _uy.u32[1];	/* y_hi */\
+		\
+			MULL32(_al,_bl,_r0);				/*printf("[1]: %10u\n",_r0);*/	/* r0 =(_al*_bl).lo   ,  no carry-out */\
+			MULH32(_al,_bl,_r1);				/*printf("[2]: %10u\n",_r1);*/	/* r1 =(_al*_bl).hi   ,  no carry-out */\
+			_r1 = __umad32_cc(_ah,_bl,_r1);		/*printf("[3]: %10u\n",_r1);*/	/* r1+=(_ah*_bl).lo   , may carry-out */\
+			_r2 = __umad32hic(_ah,_bl,  0);		/*printf("[4]: %10u\n",_r2);*/	/* r2 =(_ah*_bl).hi+cy,  no carry-out */\
+			_r1 = __umad32_cc(_al,_bh,_r1);		/*printf("[5]: %10u\n",_r1);*/	/* r1+=(_al*_bh).lo   , may carry-out */\
+			_r2 = __umad32hic_cc(_al,_bh,_r2);	/*printf("[6]: %10u\n",_r2);*/	/* r2+=(_al*_bh).hi+cy, may carry-out */\
+			_r3 = __addc(  0,  0);				/*printf("[7]: %10u\n",_r3);*/	/* r3 = cy, no carry-out */\
+			_r2 = __umad32_cc(_ah,_bh,_r2);		/*printf("[8]: %10u\n",_r2);*/	/* r2+=(_ah*_bh).lo   , may carry-out */\
+			_r3 = __umad32hic(_ah,_bh,_r3);		/*printf("[9]: %10u\n",_r3);*/	/* r3+=(_ah*_bh).hi+cy */\
+		\
+			_uy.u32[0] = _r2;\
+			_uy.u32[1] = _r3;\
+			_hi= _uy.u64;\
+		}
+		// Above test version of the MULH64 macro reveals that CY flag preservation across successive 1-line-inline-ASM calls
+		// is not guaranteed - in my tests, the crucial carry (in the sense that its loss was hosing my Mfactor self-tests)
+		// out of [5] was getting lost as written, but suddenly preserved when I added the (now-commented-out) printf statements.
+		// Having the printing on not being a practical option, we now do things the right way, namely via a single 9-instruction
+		// block of inline-ASM for the crucial MAC sequence. (Subsequently did similarly with the other MUL macros):
+		#define MULH64(_x, _y,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 _al,_ah,_bl,_bh;\
+			uint32 _r2,_r3;\
+		\
+			_ux.u64 = _x;\
+			_al = _ux.u32[0];	/* x_lo */\
+			_ah = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			_bl = _uy.u32[0];	/* y_lo */\
+			_bh = _uy.u32[1];	/* y_hi */\
+		\
+		/* Don't use %0 value but need for output-enum, so stick value into _al to get rid of 'unused	\
+		variable' compiler warning resulting from use of dedicated var 'r1' to hold %o output value: */\
+		asm volatile (\
+			"mul.hi.u32		%0,%3,%5;		\n\t"/* r1 =(r4*r6).hi   ,  no carry-out */\
+			"mad.lo.cc.u32	%0,%4,%5,%0;	\n\t"/* r1+=(r5*r6).lo   , may carry-out */\
+			"madc.hi.u32	%1,%4,%5, 0;	\n\t"/* r2 =(r5*r6).hi+cy,  no carry-out */\
+			"mad.lo.cc.u32	%0,%3,%6,%0;	\n\t"/* r1+=(r4*r7).lo   , may carry-out */\
+			"madc.hi.cc.u32	%1,%3,%6,%1;	\n\t"/* r2+=(r4*r7).hi+cy, may carry-out */\
+			"addc.u32		%2, 0, 0;		\n\t"/* r3 = cy, no carry-out */\
+			"mad.lo.cc.u32	%1,%4,%6,%1;	\n\t"/* r2+=(r5*r7).lo   , may carry-out */\
+			"madc.hi.u32	%2,%4,%6,%2;	\n\t"/* r3+=(r5*r7).hi+cy */\
+		  : "=r" (_al) "=r" (_r2) , "=r" (_r3) : "r" (_al), "r" (_ah) , "r" (_bl), "r" (_bh));\
+		\
+			_uy.u32[0] = _r2;\
+			_uy.u32[1] = _r3;\
+			_hi = _uy.u64;\
+		}
+
+	#else	// Generic (non-PTX-code) version of above:
+
+		#define MUL_LOHI64(_x,_y,_lo,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 ah_,al_,bh_,bl_;\
+			uint32 hahbh_,hahbl_,halbh_,halbl_,lahbh_,lahbl_,lalbh_,lalbl_;\
+			uint32 sumhh_,sumhl_,sumlh_,cy;\
+		\
+			_ux.u64 = _x;\
+			al_ = _ux.u32[0];	/* x_lo */\
+			ah_ = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			bl_ = _uy.u32[0];	/* y_lo */\
+			bh_ = _uy.u32[1];	/* y_hi */\
+		\
+		/* EWM: for 32-bit unsigned inputs these have a nominal latency of ?? cycles. */\
+		/* For each MUL output we include the order in which it is used in the add/carry */\
+		/* code below (6 steps, labeled (0)-(5)), in order to properly schedule the MUL: */\
+			MUL_LOHI32(ah_,bh_,lahbh_,hahbh_);	/* (x_hi*y_hi)l,h 4,2 */\
+			MUL_LOHI32(al_,bh_,lalbh_,halbh_);	/* (x_lo*y_hi)l,h 0,1 */\
+			MUL_LOHI32(ah_,bl_,lahbl_,hahbl_);	/* (x_hi*y_lo)l,h 3,1 */\
+			MUL_LOHI32(al_,bl_,lalbl_,halbl_);	/* (x_lo*y_lo)l,h -,0 */\
+		/* Bits   0- 31 : lalbl                           */\
+		/* Bits  32- 63 :*halbl + lahbl +*lalbh   (sumlh) */\
+		/* Bits  64- 95 :*hahbl +*halbh + lahbh   (sumhl) */\
+		/* Bits  96-127 : hahbh                   (sumhh) */\
+		\
+			sumlh_  = halbl_ + lalbh_;	/* (0) halbl + lalbh                       , result (partial sum) in sumlh,carryout in CY. */\
+			cy  = sumlh_ < halbl_;\
+			halbh_ += cy;\
+			cy  = halbh_ < cy;\
+			sumhl_  = hahbl_ + halbh_;	/* (1) halbh + hahbl + (carryin from sumlh), result (partial sum) in sumhl,carryout in CY. */\
+			cy += sumhl_ < hahbl_;\
+			sumhh_	= hahbh_ + cy;		/* (2)     0 + hahbh + (carryin from sumhl), result (partial sum) in sumhh,carryout in CF) bit - should be zero! */\
+		\
+			sumlh_ += lahbl_;\
+			cy  = sumlh_ < lahbl_;		/* (3) sumlh + lahbl                       , result (sumlh), carryout in CY. */\
+			lahbh_ += cy;\
+			cy  = lahbh_ < cy;\
+			sumhl_ += lahbh_;\
+			cy += sumhl_ < lahbh_;		/* (4) sumhl + lahbh + (carryin from sumlh), result (sumhl), carryout in CY. */\
+			sumhh_ += cy;				/* (5)     0 + sumhh + (carryin from sumhl), result (sumhh), carryout in CY - should be zero! [In fact assume so.] */\
+		\
+			_ux.u32[0] = sumhl_;\
+			_ux.u32[1] = sumhh_;\
+			_hi= _ux.u64;\
+			_uy.u32[0] = lalbl_;\
+			_uy.u32[1] = sumlh_;\
+			_lo= _uy.u64;\
+		}
+
+		// Compute square of uint64 x via 32-bit x = x.lo + x.hi<<32 decomposition and 32-bit arithmetic:
+		#define SQR_LOHI64(_x,_lo,_hi)\
+		{\
+			union {\
+			uint32 u32[2];\
+			uint64 u64;\
+			} a,b;\
+		\
+			uint32 ah,al,cy;\
+			uint32 hahah,lahah,hahal,lahal,halal,lalal;\
+			uint32 sumhh,sumhl,sumlh;\
+		\
+			a.u64 = _x;\
+			ah = a.u32[1];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+			al = a.u32[0];\
+		/* Compute 64-bit subproducts in (2^32.ah + al)^2 = 2^64.[ah^2] + 2^32.2.[ah.al] + [al^2]: */\
+			MULH32(ah,ah,hahah);	/* ah^2, hi half */\
+			 MULL32(ah,ah,lahah);	/* ah^2, lo half */\
+			MULH32(ah,al,hahal);	/* ah.al,hi half */\
+			 MULL32(ah,al,lahal);	/* ah.al,lo half */\
+			MULH32(al,al,halal);	/* al^2, hi half */\
+			 MULL32(al,al,lalal);	/* al^2, lo half */\
+		/* 2*<lahal,hahal>: */\
+			sumlh = lahal + lahal;\
+			sumhl = (hahal + hahal) | ((int)lahal < 0);	/* Shifts slow on most CUDA devices, so replace << 1 by add and >> 31 by signed-less-than-0 */\
+			sumhh = hahah + ((int)hahal < 0);\
+		\
+			sumlh += halal;/* (0) sumlh + halal                     , result in sumlh, carryout in CY */\
+			cy  = sumlh < halal;\
+			lahah +=    cy;/* (1) sumhl + lahah + (carryin from (0)), result in sumhl, carryout in CY */\
+			cy  = lahah < cy;\
+			sumhl += lahah;\
+			cy += sumhl < lahah;\
+			sumhh += cy;		/* (2)     0 + sumhh + (carryin from (1)), result in sumhh, carryout in CY - should be zero! (In fact assume so.) */\
+		\
+			a.u32[1] = sumhh;\
+			a.u32[0] = sumhl;\
+			_hi = a.u64;\
+			b.u32[1] = sumlh;\
+			b.u32[0] = lalal;\
+			_lo = b.u64;\
+		}
+
+		/* Generic 128-bit multiply algorithm, returning only the upper 64 bits (high part) of the result.
+		Actual calling arguments are assumed to be 64-bit ints - user must make sure this is true.
+		Result written into hi.
+		EWM: *to-do* Currently this is just MUL_LOHI64, with the _lo-arg-related stuff cut out.
+					Need to optimize, e.g. use optimizations that generate the carry into the high half
+					without requiring exact computation of the entire lower-half partial products.
+		*/
+		#define MULH64(_x, _y,_hi)\
+		{\
+			union {\
+				uint32 u32[2];	/* NVCC is little-endian, LS 32 bits in [0]. */\
+				uint64 u64;\
+			} _ux,_uy;\
+		\
+			uint32 ah_,al_,bh_,bl_;\
+			uint32 hahbh_,hahbl_,halbh_,halbl_,lahbh_,lahbl_,lalbh_;\
+			uint32 sumhh_,sumhl_,sumlh_,cy;\
+		\
+			_ux.u64 = _x;\
+			al_ = _ux.u32[0];	/* x_lo */\
+			ah_ = _ux.u32[1];	/* x_hi */\
+			_uy.u64 = _y;\
+			bl_ = _uy.u32[0];	/* y_lo */\
+			bh_ = _uy.u32[1];	/* y_hi */\
+		\
+		/* EWM: for 32-bit unsigned inputs these have a nominal latency of ?? cycles. */\
+		/* For each MUL output we include the order in which it is used in the add/carry */\
+		/* code below (6 steps, labeled (0)-(5)), in order to properly schedule the MUL: */\
+			MUL_LOHI32(ah_,bh_,lahbh_,hahbh_);	/* (x_hi*y_hi)l,h 4,2 */\
+			MUL_LOHI32(al_,bh_,lalbh_,halbh_);	/* (x_lo*y_hi)l,h 0,1 */\
+			MUL_LOHI32(ah_,bl_,lahbl_,hahbl_);	/* (x_hi*y_lo)l,h 3,1 */\
+			MULH32    (al_,bl_,       halbl_);	/* (x_lo*y_lo)l,h -,0 */\
+		/* Bits   0- 31 : lalbl                           */\
+		/* Bits  32- 63 :*halbl + lahbl +*lalbh   (sumlh) */\
+		/* Bits  64- 95 :*hahbl +*halbh + lahbh   (sumhl) */\
+		/* Bits  96-127 : hahbh                   (sumhh) */\
+		\
+			sumlh_  = halbl_ + lalbh_;	/* (0) halbl + lalbh                       , result (partial sum) in sumlh,carryout in CY. */\
+			cy  = sumlh_ < halbl_;\
+			halbh_ += cy;\
+			cy  = halbh_ < cy;\
+			sumhl_  = hahbl_ + halbh_;	/* (1) halbh + hahbl + (carryin from sumlh), result (partial sum) in sumhl,carryout in CY. */\
+			cy += sumhl_ < hahbl_;\
+			sumhh_	= hahbh_ + cy;		/* (2)     0 + hahbh + (carryin from sumhl), result (partial sum) in sumhh,carryout in CF) bit - should be zero! */\
+		\
+			sumlh_ += lahbl_;\
+			cy  = sumlh_ < lahbl_;		/* (3) sumlh + lahbl                       , result (sumlh), carryout in CY. */\
+			lahbh_ += cy;\
+			cy  = lahbh_ < cy;\
+			sumhl_ += lahbh_;\
+			cy += sumhl_ < lahbh_;		/* (4) sumhl + lahbh + (carryin from sumlh), result (sumhl), carryout in CY. */\
+			sumhh_ += cy;				/* (5)     0 + sumhh + (carryin from sumhl), result (sumhh), carryout in CY - should be zero! [In fact assume so.] */\
+		\
+			_ux.u32[0] = sumhl_;\
+			_ux.u32[1] = sumhh_;\
+			_hi= _ux.u64;\
+		}
+
+	#endif	// endif(PTX-code version of MUL_LOHI64)?
+
+   #endif	// 0 [NVCC's emulations-via-intrinsics]] | 1 [hand-rolled 64-bit-int emulations] ?
+
+  #endif	// GCC | NVCC ?
+
+	#define __MULL64(_x,_y)	({ uint64 _lo; MULL64(_x,_y,_lo); _lo; })
+	#define __MULH64(_x,_y)	({ uint64 _hi; MULH64(_x,_y,_hi); _hi; })
+
 #endif
 
 
@@ -1229,28 +1971,53 @@ or the with functions using them (if we declare no _-prepended variables local t
 #ifndef __MULL32
 	#define __MULL32(x32,y32     )	 ((uint32)(x32)*(uint32)(y32))
 #endif
-
 #ifndef __MULH32
 	#define __MULH32(x32,y32     )	(((uint32)(x32)*(uint64)(y32)) >> 32)
 #endif
-
 #ifndef MULL32
 	#define MULL32(  x32,y32,lo32)	lo32 = (uint32)((uint32)(x32)*(uint32)(y32))
 #endif
-
 #ifndef MULH32
 	#define MULH32(  x32,y32,hi32)	hi32 = __MULH32((uint32)(x32),(uint64)(y32))
 #endif
-
 #ifndef MUL_LOHI32
 	#define MUL_LOHI32(_x32,_y32,_lo,_hi)\
 	{\
-		uint64 _x = (uint64)(_x32), _y = (uint64)(_y32);\
+		uint64 _x = (uint64)(_x32) & 0x00000000FFFFFFFFull, _y = (uint64)(_y32) & 0x00000000FFFFFFFFull;\
 		uint64 _tt = _x * _y;\
 		_lo = (uint32) _tt;\
 		_hi = (uint32)(_tt >> 32);\
 	}
 #endif
+#ifndef SQR_LOHI32
+	#define SQR_LOHI32(_x,   _lo,_hi)	MUL_LOHI32(_x,_x,_lo,_hi)
+#endif
+
+#ifdef MUL_LOHI64_SUBROUTINE
+	/* 64-bit stuff below defined as functions (not macros) in imul_macro.c */
+#else
+	#ifndef __MULL64
+		#error __MULL64 Undefined!
+	#endif
+	#ifndef __MULH64
+		#error __MULH64 Undefined!
+	#endif
+	#ifndef MULL64
+		#error MULL64 Undefined!
+	#endif
+	#ifndef MULH64
+		#error MULH64 Undefined!
+	#endif
+	#ifndef MUL64x32
+		#error MUL64x32 Undefined!
+	#endif
+	#ifndef MUL_LOHI64
+		#error MUL_LOHI64 Undefined!
+	#endif
+	#ifndef SQR_LOHI64
+		#error SQR_LOHI64 Undefined!
+  #endif
+#endif
 
 #ifdef __cplusplus
 }
