@@ -1,6 +1,6 @@
 /*******************************************************************************
 *                                                                              *
-*   (C) 1997-2014 by Ernst W. Mayer.                                           *
+*   (C) 1997-2017 by Ernst W. Mayer.                                           *
 *                                                                              *
 *  This program is free software; you can redistribute it and/or modify it     *
 *  under the terms of the GNU General Public License as published by the       *
@@ -59,39 +59,19 @@ for(k=1; k <= khi; k++)	/* Do n/(radix(1)*nwt) outer loop executions...	*/
 	for(l = 0; l < 8; l++) {
 		add0 = &a[j1] + poff[l+l];	// poff[2*l] = p00,08,...,38
 		add1 = add0+p01; add2 = add0+p02; add3 = add0+p03; add4 = add0+p04; add5 = add0+p05; add6 = add0+p06; add7 = add0+p07;
-/*
-((vec_dbl*)add0)->d0 = 3;	((vec_dbl*)add0+1)->d0 = 5;
-((vec_dbl*)add1)->d0 = 1;	((vec_dbl*)add1+1)->d0 = 3;
-((vec_dbl*)add2)->d0 = 4;	((vec_dbl*)add2+1)->d0 = 5;
-((vec_dbl*)add3)->d0 = 1;	((vec_dbl*)add3+1)->d0 = 8;
-((vec_dbl*)add4)->d0 = 5;	((vec_dbl*)add4+1)->d0 = 9;
-((vec_dbl*)add5)->d0 = 9;	((vec_dbl*)add5+1)->d0 = 7;
-((vec_dbl*)add6)->d0 = 2;	((vec_dbl*)add6+1)->d0 = 9;
-((vec_dbl*)add7)->d0 = 6;	((vec_dbl*)add7+1)->d0 = 3;
-*/
 	  #ifdef USE_AVX2
 		SSE2_RADIX8_DIT_0TWIDDLE(add0,add1,add2,add3,add4,add5,add6,add7, tmp, isrt2,two)
 	  #else
 		SSE2_RADIX8_DIT_0TWIDDLE(add0,add1,add2,add3,add4,add5,add6,add7, tmp, isrt2)
 	  #endif
-/*
-printf("out0 = %20.10f, %20.10f\n", (tmp+2*0)->d0,	(tmp+2*0+1)->d0);
-printf("out1 = %20.10f, %20.10f\n", (tmp+2*1)->d0,	(tmp+2*1+1)->d0);
-printf("out2 = %20.10f, %20.10f\n", (tmp+2*2)->d0,	(tmp+2*2+1)->d0);
-printf("out3 = %20.10f, %20.10f\n", (tmp+2*3)->d0,	(tmp+2*3+1)->d0);
-printf("out4 = %20.10f, %20.10f\n", (tmp+2*4)->d0,	(tmp+2*4+1)->d0);
-printf("out5 = %20.10f, %20.10f\n", (tmp+2*5)->d0,	(tmp+2*5+1)->d0);
-printf("out6 = %20.10f, %20.10f\n", (tmp+2*6)->d0,	(tmp+2*6+1)->d0);
-printf("out7 = %20.10f, %20.10f\n", (tmp+2*7)->d0,	(tmp+2*7+1)->d0);
-exit(0);
-*/
 		tmp += 16;
 	}
 
-/*...and now do eight radix-8 subtransforms w/internal twiddles - cf. radix64_dit_pass1 for details: */
-// Note: 1st of the 15 sincos args in each call to SSE2_RADIX8_DIT_TWIDDLE_OOP is the basic isrt2 needed for
-// radix-8. This is a workaround of GCC's 30-arg limit for inline ASM macros, which proves a royal pain here.
-//...and another kludge for the 30-arg limit: put a copy of (vec_dbl)2.0 into the first of each s1p*r outputs:
+	/*...and now do eight radix-8 subtransforms w/internal twiddles - cf. radix64_dit_pass1 for details.
+	Note: 1st of the 15 sincos args in each call to SSE2_RADIX8_DIT_TWIDDLE_OOP is the basic isrt2 needed for
+	radix-8. This is a workaround of GCC's 30-arg limit for inline ASM macros, which proves a royal pain here.
+	...and another kludge for the 30-arg limit: put a copy of (vec_dbl)2.0 into the first of each s1p*r outputs:
+	*/
 	VEC_DBL_INIT(s1p01,2.0);
 	VEC_DBL_INIT(s1p02,2.0);
 	VEC_DBL_INIT(s1p03,2.0);
@@ -116,8 +96,8 @@ exit(0);
 		s1p00,s1p38,s1p30,s1p28,s1p20,s1p18,s1p10,s1p08, isrt2
 	);
   #endif
-// The with-twiddles macros have distinct impls for AVX and AVX2, but same 'prototype'
-// (No room for added 'two' arg since we are already maxed out at 30 args, and feed the 2.0 via the above kludge):
+	// The with-twiddles macros have distinct impls for AVX and AVX2, but same 'prototype'
+	// (No room for added 'two' arg since we are already maxed out at 30 args, and feed the 2.0 via the above kludge):
 	// Block 4: jt = j1 + p04;	jp = j2 + p04;
 	SSE2_RADIX8_DIT_TWIDDLE_OOP(
 		r08,r18,r28,r38,r48,r58,r68,r78,
@@ -156,32 +136,11 @@ exit(0);
 		ss4,cc4, cc6,ss6, nss2,cc2, cc3,ss3, ss1,cc1, ss7,cc7, nss5,cc5
 	);
 	// Block 7: jt = j1 + p07;	jp = j2 + p07;
-/*
-((vec_dbl*)r02)->d0 = 3;	((vec_dbl*)r02+1)->d0 = 5;
-((vec_dbl*)r12)->d0 = 1;	((vec_dbl*)r12+1)->d0 = 3;
-((vec_dbl*)r22)->d0 = 4;	((vec_dbl*)r22+1)->d0 = 5;
-((vec_dbl*)r32)->d0 = 1;	((vec_dbl*)r32+1)->d0 = 8;
-((vec_dbl*)r42)->d0 = 5;	((vec_dbl*)r42+1)->d0 = 9;
-((vec_dbl*)r52)->d0 = 9;	((vec_dbl*)r52+1)->d0 = 7;
-((vec_dbl*)r62)->d0 = 2;	((vec_dbl*)r62+1)->d0 = 9;
-((vec_dbl*)r72)->d0 = 6;	((vec_dbl*)r72+1)->d0 = 3;
-*/
 	SSE2_RADIX8_DIT_TWIDDLE_OOP(
 		r02,r12,r22,r32,r42,r52,r62,r72,	// 1/7 swap ==> r*2 / r*E swap
 		s1p07,s1p27,s1p17,s1p37,s1p0f,s1p2f,s1p1f,s1p3f,
 		ncc4,ss4, ss2,cc2, nss6,ncc6, cc7,ss7, ncc3,nss3, nss5,cc5, ss1,ncc1
 	);
-/*
-printf("out0 = %20.10f, %20.10f\n", (s1p07)->d0,	(s1p07+1)->d0);
-printf("out1 = %20.10f, %20.10f\n", (s1p27)->d0,	(s1p27+1)->d0);
-printf("out2 = %20.10f, %20.10f\n", (s1p17)->d0,	(s1p17+1)->d0);
-printf("out3 = %20.10f, %20.10f\n", (s1p37)->d0,	(s1p37+1)->d0);
-printf("out4 = %20.10f, %20.10f\n", (s1p0f)->d0,	(s1p0f+1)->d0);
-printf("out5 = %20.10f, %20.10f\n", (s1p2f)->d0,	(s1p2f+1)->d0);
-printf("out6 = %20.10f, %20.10f\n", (s1p1f)->d0,	(s1p1f+1)->d0);
-printf("out7 = %20.10f, %20.10f\n", (s1p3f)->d0,	(s1p3f+1)->d0);
-exit(0);
-*/
 
   #endif
 
@@ -232,13 +191,19 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 	if(MODULUS_TYPE == MODULUS_TYPE_MERSENNE)
 	{
 	#ifdef USE_AVX
-
+		// For AVX512-and-beyond we support only the fast Mers-carry macros.
 		add1 = &wt1[col  ];
 		add2 = &wt1[co2-1];
 		add3 = &wt1[co3-1];
-
-		l= j & (nwt-1);						tmp = half_arr + 64;	/* ptr to local storage for the doubled wtl,wtn terms: */
-		n_minus_sil  ->d0 = n-si[l  ];		tmp->d0 = wt0[    l  ];
+		/* ptr to local storage for the doubled wtl,wtn terms: */
+	  #ifdef USE_AVX512
+		tmp = half_arr +  64;	// No lookup-tables used in avx-512; instead use opmasked conditional-doubling;
+								// 1st 64 slots hold outputs of wtsinit call. Only half of said slots used in 8-way-init mode.
+	  #else
+		tmp = half_arr + 128;	// 1st 64 slots are basic-4 LUTs, next 32 are the additional 2 LOACC LUTs, next 32 hold outputs of wtsinit call
+	  #endif
+		l= j & (nwt-1);						// These rcol wts-terms are for individual-double-broadcast-to-full-vector-width,
+		n_minus_sil  ->d0 = n-si[l  ];		tmp->d0 = wt0[    l  ];	// hence the mixing of fwd/inv wts, which is normally taboo.
 		n_minus_silp1->d0 = n-si[l+1];		tmp->d1 = wt0[nwt-l  ]*scale;
 		sinwt        ->d0 = si[nwt-l  ];	tmp->d2 = wt0[    l+1];
 		sinwtm1      ->d0 = si[nwt-l-1];	tmp->d3 = wt0[nwt-l-1]*scale;
@@ -249,38 +214,161 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 		sinwt        ->d1 = si[nwt-l  ];	tmp->d2 = wt0[    l+1];
 		sinwtm1      ->d1 = si[nwt-l-1];	tmp->d3 = wt0[nwt-l-1]*scale;
 
-		l= (j+4) & (nwt-1);					++tmp;	/* Get ready for next 4 weights-related doubles... */
+		l= (j+4) & (nwt-1);					++tmp;
 		n_minus_sil  ->d2 = n-si[l  ];		tmp->d0 = wt0[    l  ];
 		n_minus_silp1->d2 = n-si[l+1];		tmp->d1 = wt0[nwt-l  ]*scale;
 		sinwt        ->d2 = si[nwt-l  ];	tmp->d2 = wt0[    l+1];
 		sinwtm1      ->d2 = si[nwt-l-1];	tmp->d3 = wt0[nwt-l-1]*scale;
 
-		l= (j+6) & (nwt-1);					++tmp;	/* Get ready for next 4 weights-related doubles... */
+		l= (j+6) & (nwt-1);					++tmp;
 		n_minus_sil  ->d3 = n-si[l  ];		tmp->d0 = wt0[    l  ];
 		n_minus_silp1->d3 = n-si[l+1];		tmp->d1 = wt0[nwt-l  ]*scale;
 		sinwt        ->d3 = si[nwt-l  ];	tmp->d2 = wt0[    l+1];
 		sinwtm1      ->d3 = si[nwt-l-1];	tmp->d3 = wt0[nwt-l-1]*scale;
+	  #ifdef USE_AVX512
+		l= (j+8) & (nwt-1);					tmp -= 3;	// Reset to same tmp-startval as above, now copy data into d4-7 slots of vec_dbl
+		n_minus_sil  ->d4 = n-si[l  ];		tmp->d4 = wt0[    l  ];
+		n_minus_silp1->d4 = n-si[l+1];		tmp->d5 = wt0[nwt-l  ]*scale;
+		sinwt        ->d4 = si[nwt-l  ];	tmp->d6 = wt0[    l+1];
+		sinwtm1      ->d4 = si[nwt-l-1];	tmp->d7 = wt0[nwt-l-1]*scale;
 
-	/* In AVX mode advance carry-ptrs just 1 for each vector-carry-macro call: */
+		l= (j+10) & (nwt-1);				++tmp;
+		n_minus_sil  ->d5 = n-si[l  ];		tmp->d4 = wt0[    l  ];
+		n_minus_silp1->d5 = n-si[l+1];		tmp->d5 = wt0[nwt-l  ]*scale;
+		sinwt        ->d5 = si[nwt-l  ];	tmp->d6 = wt0[    l+1];
+		sinwtm1      ->d5 = si[nwt-l-1];	tmp->d7 = wt0[nwt-l-1]*scale;
+
+		l= (j+12) & (nwt-1);				++tmp;
+		n_minus_sil  ->d6 = n-si[l  ];		tmp->d4 = wt0[    l  ];
+		n_minus_silp1->d6 = n-si[l+1];		tmp->d5 = wt0[nwt-l  ]*scale;
+		sinwt        ->d6 = si[nwt-l  ];	tmp->d6 = wt0[    l+1];
+		sinwtm1      ->d6 = si[nwt-l-1];	tmp->d7 = wt0[nwt-l-1]*scale;
+
+		l= (j+14) & (nwt-1);				++tmp;
+		n_minus_sil  ->d7 = n-si[l  ];		tmp->d4 = wt0[    l  ];
+		n_minus_silp1->d7 = n-si[l+1];		tmp->d5 = wt0[nwt-l  ]*scale;
+		sinwt        ->d7 = si[nwt-l  ];	tmp->d6 = wt0[    l+1];
+		sinwtm1      ->d7 = si[nwt-l-1];	tmp->d7 = wt0[nwt-l-1]*scale;
+	  #endif
+
+	  #ifdef LOACC
+
+		// Since use wt1-array in the wtsinit macro, need to fiddle this here:
+		co2 = co3;	// For all data but the first set in each j-block, co2=co3. Thus, after the first block of data is done
+					// (and only then: for all subsequent blocks it's superfluous), this assignment decrements co2 by radix(1).
+		i = (!j);
+	   #ifdef CARRY_16_WAY
+		const uint32 nloop = RADIX>>4;
+		AVX_cmplx_carry_fast_pow2_wtsinit_X16(add1,add2,add3, bjmodn, half_arr,sign_mask, n_minus_sil,n_minus_silp1,sinwt,sinwtm1, sse_bw,sse_nm1)
+	   #else
+		const uint32 nloop = RADIX>>3;
+		AVX_cmplx_carry_fast_pow2_wtsinit_X8 (add1,add2,add3, bjmodn, half_arr,sign_mask, n_minus_sil,n_minus_silp1,sinwt,sinwtm1, sse_bw,sse_nm1)
+	   #endif
+		tmp = s1p00; tm1 = cy_r; tm2 = cy_r+1; itmp = bjmodn; itm2 = bjmodn+4;
+		for(l = 0; l < nloop; l++) {
+			// Each AVX carry macro call also processes 8 prefetches of main-array data
+			add0 = a + j1 + pfetch_dist + poff[l+l];
+		#ifdef USE_AVX512
+		  #ifdef CARRY_16_WAY
+			AVX_cmplx_carry_fast_pow2_errcheck_X16(tmp, tm1,     itmp, half_arr,i,sign_mask,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03,p04);
+			tmp += 32; tm1 += 2;           itmp += 16; i = 0;
+		  #else
+			AVX_cmplx_carry_fast_pow2_errcheck_X8 (tmp, tm1,     itmp, half_arr,i,sign_mask,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03,p04);
+			tmp += 16; tm1 += 1;           itmp += 8; i = 0;
+		  #endif
+		#else
+			AVX_cmplx_carry_fast_pow2_errcheck_X8(tmp, tm1,tm2, itmp,itm2, half_arr,i,sign_mask,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03,p04);
+			tmp += 16; tm1 += 2; tm2 += 2; itmp += 8; itm2 += 8; i = 0;
+		#endif
+		}
+
+	  #else	// USE_AVX: Hi-accuracy 4-way carry is the default:
+
+		/* In AVX mode advance carry-ptrs just 1 for each vector-carry-macro call: */
 		tm1 = s1p00; tmp = cy_r; itmp = bjmodn;
-		// Each AVX carry macro call also processes 4 prefetches of main-array data
-		add0 = a + j1 + pfetch_dist;
-		AVX_cmplx_carry_norm_pow2_errcheck0_X4(tm1,add1,add2,add3,tmp,itmp,half_arr,i,n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03);
-		tm1 += 8; tmp++; itmp += 4;
-		for(l = 1; l < RADIX>>2; l++) {
+		i = (!j);
+		for(l = 0; l < RADIX>>2; l++) {
 			// Each AVX carry macro call also processes 4 prefetches of main-array data
 			add0 = a + j1 + pfetch_dist + poff[l];
-			AVX_cmplx_carry_norm_pow2_errcheck1_X4(tm1,add1,add2,add3,tmp,itmp,half_arr,  n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03);
-			tm1 += 8; tmp++; itmp += 4;
+			AVX_cmplx_carry_norm_pow2_errcheck_X4(tm1,add1,add2,add3,tmp,itmp,half_arr,i,n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03);
+			tm1 += 8; tmp++; itmp += 4; i = 0;
 		}
 
 		co2 = co3;	// For all data but the first set in each j-block, co2=co3. Thus, after the first block of data is done
 					// (and only then: for all subsequent blocks it's superfluous), this assignment decrements co2 by radix(1).
 
+	  #endif	// LOACC ?
+
 		i =((uint32)(sw - bjmodn[0]) >> 31);	/* get ready for the next set...	*/
 
 	#elif defined(USE_SSE2)
 
+	  #ifdef LOACC
+
+		uint32 k0,k1,k2,k3, ii,loop,nwtml, co2save = co2;
+
+		i = (!j);	// Need this to force 0-wod to be bigword
+		tm1 = s1p00; tmp = cy_r; tm2 = cy_r+0x01; itmp = bjmodn;
+		// Beyond chain length 8, the chained-weights scheme becomes too inaccurate, so re-init seed-wts every 8th pass:
+		for(loop = 0; loop < RADIX>>2; loop += 4)
+		{
+			ii = loop << 2;	// Reflects 4 independent carry chains being done in each SSE2_cmplx_carry_fast_pow2_errcheck call
+			/*** wt_re,wi_re,wt_im,wi_im inits. Cf. radix16_main_carry_loop.h for scalar-macro prototyping of this: ***/
+			l = j & (nwt-1);	nwtml = nwt-l;
+			n_minus_sil   = n-si[l  ];
+			n_minus_silp1 = n-si[l+1];
+			sinwt   = si[nwtml  ];
+			sinwtm1 = si[nwtml-1];
+			wtl     = wt0[    l  ];
+			wtn     = wt0[nwtml  ]*scale;
+			wtlp1   = wt0[    l+1];
+			wtnm1   = wt0[nwtml-1]*scale;
+
+			co2 = co2save;	// Need this for all wts-inits beynd the initial set, due to the co2 = co3 preceding the (j+2) data
+			ctmp = (struct complex *)half_arr + 24;	// ptr to local storage for the doubled wtl,wtn terms:
+			// (j)-data occupy the 8 xmm-sized slots above the 16 used by fixed auxiliary-data, and overwrite these inits:
+			ctmp->re = ctmp->im = wtl;		ctmp += 2;
+			ctmp->re = ctmp->im = wtn;		ctmp += 2;
+			ctmp->re = ctmp->im = wtlp1;	ctmp += 2;
+			ctmp->re = ctmp->im = wtnm1;
+
+			l = (j+2) & (nwt-1);	nwtml = nwt-l;;
+			k0 = n-si[l  ];
+			k1 = n-si[l+1];
+			k2 = si[nwtml  ];
+			k3 = si[nwtml-1];
+			wtl     = wt0[    l  ];
+			wtn     = wt0[nwtml  ]*scale;
+			wtlp1   = wt0[    l+1];
+			wtnm1   = wt0[nwtml-1]*scale;
+
+			ctmp = (struct complex *)half_arr + 32;	// (j+2) data start at ctmp + 8
+			ctmp->re = ctmp->im = wtl;		ctmp += 2;
+			ctmp->re = ctmp->im = wtn;		ctmp += 2;
+			ctmp->re = ctmp->im = wtlp1;	ctmp += 2;
+			ctmp->re = ctmp->im = wtnm1;
+
+			add1 = &wt1[col  +ii];	/* Don't use add0 here, to avoid need to reload main-array address */
+			add2 = &wt1[co2-1-ii];
+			add3 = &wt1[co3-1-ii];
+
+			// Since use wt1-array in the wtsinit macro, need to fiddle this here:
+			co2 = co3;	// For all data but the first set in each j-block, co2=co3. Thus, after the first block of data is done
+						// (and only then: for all subsequent blocks it's superfluous), this assignment decrements co2 by radix(1).
+			// *But*: since the init macro does an on-the-fly version of this between j,j+2 portions, external code co2=co3 must come *after* both ctmp-data octets are inited.
+			add0 = (double*)(bjmodn+ii);
+			SSE2_cmplx_carry_fast_pow2_wtsinit(add1,add2,add3, add0, half_arr,sign_mask, n_minus_sil,n_minus_silp1,sinwt,sinwtm1, k0,k1,k2,k3, sse_bw,sse_nm1)
+
+			for(l = loop; l < loop+4; l++) {
+				// Each SSE2 LOACC carry macro call also processes 4 prefetches of main-array data:
+				add0 = a + j1 + pfetch_dist + poff[l];	// poff[] = p0,4,8,...
+				SSE2_cmplx_carry_fast_pow2_errcheck(tm1,tmp,tm2,itmp,half_arr,i,sign_mask,sse_bw,sse_nm1,sse_sw, add0,p01,p02,p03);
+				tm1 += 8; tmp += 2; tm2 += 2; itmp += 4; i = 0;
+			}
+		}
+
+	  #else	// Hi-accuracy is the default:
+
 		l= j & (nwt-1);			/* We want (S*J mod N) - SI(L) for all 32 carries, so precompute	*/
 		n_minus_sil   = n-si[l  ];		/* N - SI(L) and for each J, find N - (B*J mod N) - SI(L)		*/
 		n_minus_silp1 = n-si[l+1];		/* For the inverse weight, want (S*(N - J) mod N) - SI(NWT - L) =	*/
@@ -303,14 +391,13 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 		add3 = &wt1[co3-1];
 
 		tm1 = s1p00; tmp = cy_r; tm2 = cy_r+0x01; itmp = bjmodn;
-		// Each SSE2 carry macro call also processes 2 prefetches of main-array data
-		add0 = a + j1 + pfetch_dist;
-		SSE2_cmplx_carry_norm_pow2_errcheck0_2B(tm1,add1,add2,add3,tmp,tm2,itmp,half_arr,i,n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01);	tm1 += 8; tmp += 2; tm2 += 2; itmp += 4;
-		for(l = 1; l < RADIX>>2; l++) {
+		i = (!j);
+		for(l = 0; l < RADIX>>2; l++) {
 			// Each SSE2 carry macro call also processes 2 prefetches of main-array data
 			add0 = a + j1 + pfetch_dist + poff[l];	// poff[] = p0,4,8,...
 			add0 += (-(l&0x1)) & p02;	// Base-addr incr by extra p2 on odd-index passes
-			SSE2_cmplx_carry_norm_pow2_errcheck1_2B(tm1,add1,add2,add3,tmp,tm2,itmp,half_arr,  n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01);	tm1 += 8; tmp += 2; tm2 += 2; itmp += 4;
+			SSE2_cmplx_carry_norm_pow2_errcheck1_2B(tm1,add1,add2,add3,tmp,tm2,itmp,half_arr,i,n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p01);
+			tm1 += 8; tmp += 2; tm2 += 2; itmp += 4; i = 0;
 		}
 
 		l= (j+2) & (nwt-1);			/* We want (S*J mod N) - SI(L) for all 16 carries, so precompute	*/
@@ -340,9 +427,12 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 			// Each SSE2 carry macro call also processes 2 prefetches of main-array data
 			add0 = a + j1 + pfetch_dist + poff[l];	// poff[] = p0,4,8,...
 			add0 += (-(l&0x1)) & p02;	// Base-addr incr by extra p2 on odd-index passes
-			SSE2_cmplx_carry_norm_pow2_errcheck2_2B(tm1,add1,add2,     tmp,tm2,itmp,half_arr,  n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p02,p03);	tm1 += 8; tmp += 2; tm2 += 2; itmp += 4;
+			SSE2_cmplx_carry_norm_pow2_errcheck2_2B(tm1,add1,add2,     tmp,tm2,itmp,half_arr,  n_minus_silp1,n_minus_sil,sign_mask,sinwt,sinwtm1,sse_bw,sse_nm1,sse_sw, add0,p02,p03);
+			tm1 += 8; tmp += 2; tm2 += 2; itmp += 4;
 		}
 
+	  #endif // HIACC or LOACC?
+
 		i =((uint32)(sw - bjmodn[0]) >> 31);	/* get ready for the next set...	*/
 
 	#else	// Scalar-double mode:
@@ -358,22 +448,43 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 		wtlp1   =wt0[    l+1];
 		wtnm1   =wt0[nwt-l-1]*scale;	/* ...and here.	*/
 
-		/*...set0 is slightly different from others; divide work into 16 blocks of 4 macro calls, 1st set of which gets pulled out of loop: */
+	  #ifdef LOACC
 
+		/*...set0 is slightly different from others; divide work into blocks of 4 macro calls, 1st set of which gets pulled out of loop: */
 		l = 0; addr = cy_r; itmp = bjmodn;
-	   cmplx_carry_norm_pow2_errcheck0(a[j1    ],a[j2    ],*addr,*itmp  ); ++l; ++addr; ++itmp;
+	   cmplx_carry_norm_pow2_errcheck0(a[j1    ],a[j2    ],*addr,*itmp,0); ++l; ++addr; ++itmp;
+		cmplx_carry_fast_pow2_errcheck(a[j1+p01],a[j2+p01],*addr,*itmp,l); ++l; ++addr; ++itmp;
+		cmplx_carry_fast_pow2_errcheck(a[j1+p02],a[j2+p02],*addr,*itmp,l); ++l; ++addr; ++itmp;
+		cmplx_carry_fast_pow2_errcheck(a[j1+p03],a[j2+p03],*addr,*itmp,l); ++l; ++addr; ++itmp;
+		// Remaining quartets of macro calls done in loop:
+		for(ntmp = 1; ntmp < RADIX>>2; ntmp++) {
+			jt = j1 + poff[ntmp]; jp = j2 + poff[ntmp];	// poff[] = p04,08,...
+			// Re-init weights every 4th macro invocatin to keep errors under control:
+			cmplx_carry_norm_pow2_errcheck0(a[jt    ],a[jp    ],*addr,*itmp,l); ++l; ++addr; ++itmp;
+			cmplx_carry_fast_pow2_errcheck (a[jt+p01],a[jp+p01],*addr,*itmp,l); ++l; ++addr; ++itmp;
+			cmplx_carry_fast_pow2_errcheck (a[jt+p02],a[jp+p02],*addr,*itmp,l); ++l; ++addr; ++itmp;
+			cmplx_carry_fast_pow2_errcheck (a[jt+p03],a[jp+p03],*addr,*itmp,l); ++l; ++addr; ++itmp;
+		}
+
+	  #else	// Hi-accuracy is the default:
+
+		/*...set0 is slightly different from others; divide work into blocks of 4 macro calls, 1st set of which gets pulled out of loop: */
+		l = 0; addr = cy_r; itmp = bjmodn;
+	   cmplx_carry_norm_pow2_errcheck0(a[j1    ],a[j2    ],*addr,*itmp,0); ++l; ++addr; ++itmp;
 		cmplx_carry_norm_pow2_errcheck(a[j1+p01],a[j2+p01],*addr,*itmp,l); ++l; ++addr; ++itmp;
 		cmplx_carry_norm_pow2_errcheck(a[j1+p02],a[j2+p02],*addr,*itmp,l); ++l; ++addr; ++itmp;
 		cmplx_carry_norm_pow2_errcheck(a[j1+p03],a[j2+p03],*addr,*itmp,l); ++l; ++addr; ++itmp;
 		// Remaining quartets of macro calls done in loop:
 		for(ntmp = 1; ntmp < RADIX>>2; ntmp++) {
-			jt = j1 + poff[ntmp]; jp = j2 + poff[ntmp];	// poff[] = p04,08,...,60
+			jt = j1 + poff[ntmp]; jp = j2 + poff[ntmp];	// poff[] = p04,08,...
 			cmplx_carry_norm_pow2_errcheck(a[jt    ],a[jp    ],*addr,*itmp,l); ++l; ++addr; ++itmp;
 			cmplx_carry_norm_pow2_errcheck(a[jt+p01],a[jp+p01],*addr,*itmp,l); ++l; ++addr; ++itmp;
 			cmplx_carry_norm_pow2_errcheck(a[jt+p02],a[jp+p02],*addr,*itmp,l); ++l; ++addr; ++itmp;
 			cmplx_carry_norm_pow2_errcheck(a[jt+p03],a[jp+p03],*addr,*itmp,l); ++l; ++addr; ++itmp;
 		}
 
+	  #endif
+
 		i =((uint32)(sw - bjmodn[0]) >> 31);	/* get ready for the next set...	*/
 		co2=co3;	/* For all data but the first set in each j-block, co2=co3. Thus, after the first block of data is done
 				 and only then: for all subsequent blocks it's superfluous), this assignment decrements co2 by radix(1).	*/
@@ -382,7 +493,96 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 	}
 	else	/* Fermat-mod carry in SIMD mode */
 	{
-	#ifdef USE_AVX
+	#ifdef USE_AVX512
+
+		// For a description of the data movement in AVX mode, see radix28_ditN_cy_dif1.
+		tmp = half_arr+2;	// In Fermat-mod mode, use first 2 vector slots above half_arr for base and 1/base
+		VEC_DBL_INIT(tmp, scale);
+		/* Get the needed Nth root of -1: */
+		add1 = (double *)&rn0[0];
+		add2 = (double *)&rn1[0];
+
+		idx_offset = j;
+		idx_incr = NDIVR;
+
+		tmp = base_negacyclic_root;	tm2 = tmp+1;
+
+		// Hi-accuracy version needs 2 copies of each base root, one for each invocation of the SSE2_fermat_carry_norm_pow2 carry macri:
+		l = (j >> 1);	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+		tmp += 2;	tm2 += 2;
+		l += 1;	k1=(l & NRTM1);	k2=(l >> NRT_BITS);
+		dtmp=rn0[k1].re;			wt_im=rn0[k1].im;
+		rt  =rn1[k2].re;			it   =rn1[k2].im;
+		wt_re =dtmp*rt-wt_im*it;	wt_im =dtmp*it+wt_im*rt;
+		for(i = 0; i < (RADIX << 1); i += 16) {
+			VEC_DBL_INIT(tmp+ i,wt_re);	VEC_DBL_INIT(tm2+ i,wt_im);
+		}
+
+		// AVX-custom 8-way carry macro - each contains 8 of the [RADIX] stride-n/RADIX-separated carries
+		// (processed independently in parallel), and steps through sequential-data indicies j,j+2,j+4,j+6,j+8,j+10,j+12,j+14:
+		tm0 = s1p00; tmp = base_negacyclic_root; tm1 = cy_r; tm2 = cy_i; l = 0x2000;
+		for(i = 0; i < RADIX>>3; i++) {	// RADIX/8 loop passes
+			// Each AVX carry macro call also processes 8 prefetches of main-array data
+			add0 = a + j1 + pfetch_dist + poff[2*i];	// Can't use tm2 as prefetch base-ptr for pow2 Fermat-mod carry-macro since that's used for cy_i
+			SSE2_fermat_carry_norm_pow2_errcheck_X8(tm0,tmp,l,tm1,tm2,half_arr,sign_mask, add0,p01,p02,p03,p04);
+			tm0 += 16; tm1++; tm2++; tmp += 16; l -= 0x380;
+		}
+
+	#elif defined(USE_AVX)
 
 		// For a description of the data movement for Fermat-mod carries in SSE2 mode, see radix16_ditN_cy_dif1.c.
 		// For a description of the data movement in AVX mode, see radix28_ditN_cy_dif1.
@@ -432,7 +632,7 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 		}
 
 		// AVX-custom 4-way carry macro - each contains 4 of the RADIX stride-n/RADIX-separated carries
-		// (processed independently in parallel), and steps through sequential-data indices j,j+2,j+4,j+6:
+		// (processed independently in parallel), and steps through sequential-data indicies j,j+2,j+4,j+6:
 
 	// The starting value of the literal pointer offsets following 'tmp' in these macro calls = RADIX*2*sizeof(vec_dbl) = 2^12
 	// which is the byte offset between the 'active' negacyclic weights [pointed to by base_negacyclic_root] and the
@@ -464,7 +664,7 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 	  #if (OS_BITS == 32)
 		for(l = 0; l < RADIX; l++) {	// RADIX loop passes
 			// Each SSE2 carry macro call also processes 1 prefetch of main-array data
-			tm2 = a + j1 + pfetch_dist + poff[l];	// poff[] = p0,4,8,...
+			tm2 = a + j1 + pfetch_dist + poff[l>>2];	// poff[] = p0,4,8,...
 			tm2 += (-(l&0x10)) & p02;
 			tm2 += (-(l&0x01)) & p01;
 			SSE2_fermat_carry_norm_pow2_errcheck   (tm1,tmp,NRT_BITS,NRTM1,idx_offset,idx_incr,half_arr,sign_mask,add1,add2, tm2);
@@ -473,7 +673,7 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 	  #else	// 64-bit SSE2
 		for(l = 0; l < RADIX>>1; l++) {	// RADIX/2 loop passes
 			// Each SSE2 carry macro call also processes 2 prefetches of main-array data
-			tm2 = a + j1 + pfetch_dist + poff[l];	// poff[] = p0,4,8,...; (tm1-cy_r) acts as a linear loop index running from 0,...,RADIX-1 here.
+			tm2 = a + j1 + pfetch_dist + poff[l>>1];	// poff[] = p0,4,8,...; (tm1-cy_r) acts as a linear loop index running from 0,...,RADIX-1 here.
 			tm2 += (-(l&0x1)) & p02;	// Base-addr incr by extra p2 on odd-index passes
 			SSE2_fermat_carry_norm_pow2_errcheck_X2(tm1,tmp,NRT_BITS,NRTM1,idx_offset,idx_incr,half_arr,sign_mask,add1,add2, tm2,p01);
 			tm1 += 4; tmp += 2;
@@ -485,7 +685,7 @@ normally be getting dispatched to [radix] separate blocks of the A-array, we nee
 		// Can't use l as loop index here, since it gets used in the Fermat-mod carry macro (as are k1,k2);
 		ntmp = 0; addr = cy_r; addi = cy_i;
 		for(m = 0; m < RADIX>>2; m++) {
-			jt = j1 + poff[m]; jp = j2 + poff[m];	// poff[] = p04,08,...,60
+			jt = j1 + poff[m]; jp = j2 + poff[m];	// poff[] = p04,08,...
 			fermat_carry_norm_pow2_errcheck(a[jt    ],a[jp    ],*addr,*addi,ntmp,NRTM1,NRT_BITS);	ntmp += NDIVR; ++addr; ++addi;
 			fermat_carry_norm_pow2_errcheck(a[jt+p01],a[jp+p01],*addr,*addi,ntmp,NRTM1,NRT_BITS);	ntmp += NDIVR; ++addr; ++addi;
 			fermat_carry_norm_pow2_errcheck(a[jt+p02],a[jp+p02],*addr,*addi,ntmp,NRTM1,NRT_BITS);	ntmp += NDIVR; ++addr; ++addi;
@@ -520,14 +720,30 @@ in the same order here as DIF, but the in-and-output-index offsets are BRed: j1
 
   #else
 
-   #ifdef USE_AVX
-	#define OFF1	0x200
-	#define OFF2	0x400
-	#define OFF3	0x600
-	#define OFF4	0x800
-	#define OFF5	0xa00
-	#define OFF6	0xc00
-	#define OFF7	0xe00
+   #ifdef USE_ARM_V8_SIMD
+	const uint32 OFF1 = 0x100;
+	const uint32 OFF2 = 0x200;
+	const uint32 OFF3 = 0x300;
+	const uint32 OFF4 = 0x400;
+	const uint32 OFF5 = 0x500;
+	const uint32 OFF6 = 0x600;
+	const uint32 OFF7 = 0x700;
+   #elif defined(USE_AVX512)
+	#define OFF1	4*0x100
+	#define OFF2	4*0x200
+	#define OFF3	4*0x300
+	#define OFF4	4*0x400
+	#define OFF5	4*0x500
+	#define OFF6	4*0x600
+	#define OFF7	4*0x700
+   #elif defined(USE_AVX)
+	#define OFF1	2*0x100
+	#define OFF2	2*0x200
+	#define OFF3	2*0x300
+	#define OFF4	2*0x400
+	#define OFF5	2*0x500
+	#define OFF6	2*0x600
+	#define OFF7	2*0x700
    #else
 	#define OFF1	0x100
 	#define OFF2	0x200
@@ -539,7 +755,7 @@ in the same order here as DIF, but the in-and-output-index offsets are BRed: j1
    #endif
 
    // Unlike for DIT, reducing obj-code size here by wrapping SSE2_RADIX8_DIF_0TWIDDLE in a loop hurt performance on SSE2:
-   #if 0
+   #if 1	// Jan 2016: Loop version sign. faster on KNL under AVX-512, make that the default moving forward
 
 	// Outout (s-pointer) offsets are normal BRed - note r's in this routine are separated as 1*index-stride, not 2* as for others:
 	vec_dbl *out0, *out1, *out2, *out3, *out4, *out5, *out6, *out7;
@@ -547,32 +763,11 @@ in the same order here as DIF, but the in-and-output-index offsets are BRed: j1
 	for(l = 0; l < 8; l++) {
 		k1 = reverse(l,8)<<1;
 		tmp = s1p00 + k1;
-/*
-(tmp + 0x00)->d0 = 3;	(tmp+1+0x00)->d0 = 5;
-(tmp + 0x10)->d0 = 1;	(tmp+1+0x10)->d0 = 3;
-(tmp + 0x20)->d0 = 4;	(tmp+1+0x20)->d0 = 5;
-(tmp + 0x30)->d0 = 1;	(tmp+1+0x30)->d0 = 8;
-(tmp + 0x40)->d0 = 5;	(tmp+1+0x40)->d0 = 9;
-(tmp + 0x50)->d0 = 9;	(tmp+1+0x50)->d0 = 7;
-(tmp + 0x60)->d0 = 2;	(tmp+1+0x60)->d0 = 9;
-(tmp + 0x70)->d0 = 6;	(tmp+1+0x70)->d0 = 3;
-*/
 	  #ifdef USE_AVX2
 		SSE2_RADIX8_DIF_0TWIDDLE(
 			tmp,OFF1,OFF2,OFF3,OFF4,OFF5,OFF6,OFF7,
 			out0,out1,out2,out3,out4,out5,out6,out7, isrt2,two
 		);
-/*
-printf("out0 = %20.10f, %20.10f\n", (out0)->d0,	(out0+1)->d0);
-printf("out1 = %20.10f, %20.10f\n", (out1)->d0,	(out1+1)->d0);
-printf("out2 = %20.10f, %20.10f\n", (out2)->d0,	(out2+1)->d0);
-printf("out3 = %20.10f, %20.10f\n", (out3)->d0,	(out3+1)->d0);
-printf("out4 = %20.10f, %20.10f\n", (out4)->d0,	(out4+1)->d0);
-printf("out5 = %20.10f, %20.10f\n", (out5)->d0,	(out5+1)->d0);
-printf("out6 = %20.10f, %20.10f\n", (out6)->d0,	(out6+1)->d0);
-printf("out7 = %20.10f, %20.10f\n", (out7)->d0,	(out7+1)->d0);
-exit(0);
-*/
 	  #else
 		SSE2_RADIX8_DIF_0TWIDDLE(
 			tmp,OFF1,OFF2,OFF3,OFF4,OFF5,OFF6,OFF7,
@@ -615,10 +810,10 @@ exit(0);
 
     #endif	// 0?
 
-/*...and now do eight radix-8 subtransforms w/internal twiddles - cf. radix64_dif_pass1 for details: */
-// Note: 1st of the 15 sincos args in each call to SSE2_RADIX8_DIT_TWIDDLE_OOP is the basic isrt2 needed for
-// radix-8. This is a workaround of GCC's 30-arg limit for inline ASM macros, which proves a royal pain here.
-
+/*...and now do eight radix-8 subtransforms w/internal twiddles - cf. radix64_dif_pass1 for details.
+Note: 1st of the 15 sincos args in each call to SSE2_RADIX8_DIT_TWIDDLE_OOP is the basic isrt2 needed for
+radix-8. This is a workaround of GCC's 30-arg limit for inline ASM macros, which proves a royal pain here.
+*/
 	/* Block 0: */
 	add0 = &a[j1]      ; add1 = add0+p01; add2 = add0+p02; add3 = add0+p03; add4 = add0+p04; add5 = add0+p05; add6 = add0+p06; add7 = add0+p07;
 	/* 0-index block has all-unity twiddles: Remember, the twiddleless DIF bit-reverses both its in-and-outputs,
@@ -692,6 +887,7 @@ exit(0);
 		ncc4,ss4, ss2,cc2, nss6,ncc6, cc7,ss7, ncc3,nss3, nss5,cc5, ss1,ncc1
 	);
 
+   #ifndef USE_ARM_V8_SIMD
 	#undef OFF1
 	#undef OFF2
 	#undef OFF3
@@ -699,6 +895,7 @@ exit(0);
 	#undef OFF5
 	#undef OFF6
 	#undef OFF7
+   #endif
 
   #endif	// USE_SCALAR_DFT_MACRO?
 
